(define-module aws
  (extend aws.error)
  (use gauche.parameter)
  (use gauche.sequence)
  (use rfc.http)
  (use rfc.uri)
  (use srfi-13)
  (use text.inflection)
  (export
   *aws-documentation-host-name*
   aws-developer-guide
   *aws-batch-request-max*
   *aws-pagination-max*
   *aws-request-num-chars-max*
   aws-endpoint-host-name
   aws-xslt-endpoint-host-name
   aws-access-key-id
   *aws-locales*
   aws-locale
   <aws-rest-query>
   query->uri execute
   )
  )

(select-module aws)

(define-macro (export-with-prefix-aws name)
  (let ((name-with-prefix (string->symbol (format #f "aws-~a" name))))
    `(begin
       (define ,name-with-prefix ,name)
       (export ,name-with-prefix))))

(define-macro (define-aws-enum . identifies)
  `(begin
     ,@(map-with-index
        (lambda (i id)
          `(define ,(string->symbol (format #f "*aws-~a*" id)) ,i))
        identifies)))

(define-syntax define-by-locale
  (syntax-rules ()
    ((_ name options)
     (define (name)
       (and-let* ((pair (assq (aws-locale) options)))
         (cdr pair))))))

(define *aws-documentation-host-name* "docs.amazonwebservices.com")

(define (aws-developer-guide path)
  (string-append "/AWSECommerceService/2007-10-29/DG" path))

;;; Reference: 
;;; [1] Amazon Associates Web Service Developer Guide (API Version 2007-10-29)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Making Requests and Understanding Responses
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define *aws-batch-request-max* 10)

;;; Batch Requests
;;
;; Parameters That Differ:
;;  OperationName.ReferenceNumber.Parameter=Value
;; where
;;  ReferenceNumber: a positive integer
;;
;; Shared parameters:
;;  OperationName.Shared.Parameter=value

;;; Multiple Operation Requests
;;
;; - The value of Operation is set to two, comma-separated operations.
;; - Multiple operations requests can have only two different operations.

;;; Required Parameters
;;
;; Service: AWSECommerceService
;; AWSAccessKeyId: `Your Amazon-assigned Access Key ID'
;; (or SubscriptionId: `Your Amazon-assigned subscription ID')
;; Operation: `Operation you wish to perform, e.g., ItemLookup'

(define aws-access-key-id (make-parameter #f))

;;; General, Optional Parameters
;;
;; AssociateTag: `An Amazon-assigned Associate ID'
;; MerchantId: `An alphanumeric token assigned by Amazon to a merchant'
;; ResponseGroup: `Various'
;; Version: `Various` (Version of the A2S 4.0 WSDL. The default is 2005-10-05.)

;;; XSL Parameters
;;
;; Style: "XML", the default, or the URL of an XSL stylesheet (REST requests only)
;; ContentType: "text/xml", the default, or "text/html" (REST requests only)

;;; XML-Encoding Parameters
;;
;; XMLEscaping: "Single", the default, or "Double"

;;; Debugging Parameters
;;
;; Validate: "False", the default, or "True"

;;; Request response Group
;;
;; It echoes back parameters used to the response. Helpful for debugging.

;;; Help Operation
;;
;; example:
;; http://ecs.amazonaws.com/onca/xml?
;; Service=AWSECommerceService&
;; AWSAccessKeyId=[AWS Access Key ID]&
;; Operation=Help&
;; HelpType=Operation&
;; About=CartAdd
;;
;; - HelpType is either "Operation" or "ResponseGroup."
;; - The About parameter is set to an A2S operation or response group.

;;; Paging Through Results
;;
;; example:
;;  request: ItemPage
;;  response: ItemPage, TotalResults, TotalPages

;;; Maximum Number Of Returned Pages

(define *aws-pagination-max*
  '((item-lookup variation-page 150)
    (item-lookup review-page 20)
    (item-lookup offer-page 100)
    (item-search item-page 400)
    (list-lookup product-page 30)
    (list-search list-page 20)
    (customer-content-lookup review-page 10)
    (customer-content-search customer-page 20)
    ))

;;; Sorting Results
;;
;; example:
;; http://ecs.amazonaws.com/onca/xml?
;; Service=AWSECommerceService&
;; AWSAccessKeyId=1MMR0DKBKVGCHE7TZBG2&
;; Operation=ItemSearch&
;; Keywords=Harry%20Potter&
;; SearchIndex=Books&
;; Sort=titlerank&
;; ItemPage=29&
;; Version=2006-09-13
;;
;; - Sort parameters also differ by locale.
;; - See the appendix for more information about sort value by locale.
;; - Default Sort Values:
;;    ItemSearch with BrowseNode: sort by Relevance
;;    ItemSearch without BrowseNode: sort by BestSeller ranking.

;;; XSLT Service URLs

(define *aws-xslt-endpoint-host-names*
  '((ca . "xml-ca.amznxslt.com")
    (de . "xml-de.amznxslt.com")
    (fr . "xml-fr.amznxslt.com")
    (jp . "xml-jp.amznxslt.com")
    (uk . "xml-uk.amznxslt.com")
    (us . "xml-us.amznxslt.com")
    ))

(define-by-locale aws-xslt-endpoint-host-name *aws-xslt-endpoint-host-names*)

;;; Creating an XSL Style Sheet
;;
;; example xsl:
;; http://ecs.amazonaws.com/xsl/aws4/item-search.xsl

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Making Requests and Understanding Responses > Making REST Requests
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; General Request Format

(define *aws-rest-fixed-parameters* ; Locale-independent
  '((service . "AWSECommerceService"))
  )

(define *aws-endpoint-host-names*
  '((ca . "ecs.amazonaws.ca")
    (de . "ecs.amazonaws.de")
    (fr . "ecs.amazonaws.fr")
    (jp . "ecs.amazonaws.jp")
    (uk . "ecs.amazonaws.co.uk")
    (us . "ecs.amazonaws.com")
    ))

(define-by-locale aws-endpoint-host-name *aws-endpoint-host-names*)

(define *aws-endpoint-path-prefix* "/onca/xml") ; Locale-independent

;;; Request Limitations
;;
;; `The limit for a URL in Microsoft's Internet Explorer is a little more than 2000 characters.'
(define *aws-request-num-chars-max* 2000)

;;; REST Syntax
;;
;; - Parameter values must be URL-encoded.
;; - Parameter names and values are case sensitive.
;; - Compound Parameters

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Making Requests and Understanding Responses > Understanding Responses
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; For REST requests, this data structure is simply the body of the HTTP response.
;; You can use a data binding method for REST responses, or use an XML parser directly
;; to process the information.

;;; Anatomy of a Response
;; Version Information Segment:
;;   the first two lines of every response contain version information.
;; Operation Request Segment:
;;   - the arguments, or operation parameters (includes ReqestID)
;;   - the processing time
;; Request Validation Segment:
;;   the reasons why your requests fail:
;; Item Attributes Segment:
;;   - the final segment of the response that contains the item attributes is
;;     typically the part of the response that contains the most value.
;;   - The item attributes returned in a response vary according to
;;     the response group used and the items found.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Finding Items to Buy > Search Operations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; A2S offers the following operations to find items:

(define-macro (define-operation-to-find-items . identifies)
  `(define-aws-enum ,@identifies))

(define-operation-to-find-items
  item-search
  seller-listing-search
  list-search
  list-lookup
  browse-node-lookup
  )

;; If you do not know whether an item is for sale by a merchant or seller, search first using ItemSearch for two reasons: 
;; - Most of the items sold on Amazon are sold by merchants
;; - Sellers often list their items for sale in Amazon's marketplace as well as in their own stores.
;;   Anything sold in the marketplace can be found by ItemSearch.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Finding Items to Buy > Items You Cannot Buy
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; The best way to determine whether or not an item can really be purchased is
;; by using one of the following response groups in a request:
;; - Offers
;; - OfferFull
;; - Large, which is a parent of the Offers response group.

;; - All of the search operations listed above, *except* BrowseNodeLookup, can use these response groups.
;; - Items that do *not* have an OfferListingId cannot be purchased.
;; - The Availability element's message concerns how quickly an item can ship *when* it is in stock.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Finding Items to Buy > Finding Items Sold by Merchants
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; - ItemSearch is one of the most heavily used A2S operations.
;; - Each search index supports only a subset of all ItemSearch parameters.
;; - The search index values that can be used in an ItemSearch request vary by locale.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Finding Items to Buy > Using Search Bins to Find Items
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; - The SearchBins response group categorizes the items returned by ItemSearch into groups, called bins.
;; - The response would return multiple sets of bins, called SearchBinSets, in which
;;   the items would be divided according to different criteria.
;; - The criteria used to divide the returned items into bins is called the NarrowBy value.
;; - Search bins are only available in the US locale.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Finding Items to Buy > Finding Items Sold by Sellers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; - Merchants always sell new or collectible items.
;; - You must purchase items that are used, refurbished, or out of stock in merchant's inventories, from sellers.
;; - Sellers also can sell new and collectible items.
;; - The A2S operation used to find items sold by sellers is SellerListingSearch.

;;; Using SellerListingSearch
;; example:
;; http://ecs.amazonaws.com/onca/xml?
;; Service=AWSECommerceService&
;; AWSAccessKeyId=[AWS Access Key ID]&
;; Operation=SellerListingSearch&
;; SellerId=[A Seller ID]&
;; Keywords=crystal%20ball&
;; Sort=+price

;; - Word order in the title does not matter and case is not significant.
;; - A2S does not offer an operation that searches for a specific item across all sellers.

;;; Finding Seller IDs

;;; Searching for Out of Stock Items
;; To search for discontinued, out of stock, or unavailable products,
;; set the SellerListingSearch parameter, OfferStatus, to "Closed."
;; example:
;; http://ecs.amazonaws.com/onca/xml?
;; Service=AWSECommerceService
;;  &AWSAccessKeyId=[ID]
;;  &Operation=SellerListingSearch
;;  &SellerId=[Seller  ID]
;;  &Title=[GameTitle]
;;  &OfferStatus=Closed

;;; Returning Historical Pricing

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Finding Items to Buy > Finding Items Using Browse Nodes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; - Browse nodes form an organizational hierarchy in which Amazon places items for sale.
;; - Each browse node has an ID, which is a positive integer, and a name.
;; - ItemSearch returns all of the items associated with a node and all of the nodes below it.

;; example:
;; http://ecs.amazonaws.com/onca/xml?
;; Service=AWSECommerceService&
;; AWSAccessKeyId=[AWS Access Key ID]&
;; Operation=ItemSearch&
;; Keywords=tall&
;; SearchIndex=Apparel&
;; BrowseNode=123456

;;; Traversing the Browse Node Hierarchy
;; BrowseNodeLookup operation is the most direct way of traversing the the browse node hierarchy.

;; Look at the response to the following request:
;; http://ecs.amazonaws.com/onca/xml?
;; Service=AWSECommerceService&
;; AWSAccessKeyId=[Access Key ID&
;; Operation=BrowseNodeLookup&
;; BrowseNodeId=11232&
;; ResponseGroup=BrowseNodeInfo

;; A snippet of its response is:
;; <Item>
;; <ASIN>0131856340</ASIN>
;; <BrowseNodes>
;;   <BrowseNode>
;;   <BrowseNodeId>11232</BrowseNodeId>
;;   <Name> Social Sciences</Name>
;;   <Ancestors>
;;     <BrowseNode>
;;       <BrowseNodeId>53</BrowseNodeId>
;;       <Name>Nonfiction</Name>
;;       <Ancestors>
;;         <BrowseNode>
;;           <BrowseNodeId>1000</BrowseNodeId>
;;           <Name>Subjects</Name>
;;           <Ancestors>
;;             <BrowseNode>
;;               <BrowseNodeId>283155</BrowseNodeId>
;;               <Name>Books</Name>
;;             </BrowseNode>
;;           </Ancestors>
;;         </BrowseNode>
;;       </Ancestors>
;;     </BrowseNode>
;;   </Ancestors>
;;   <Children>
;;     <BrowseNode>
;;       <BrowseNodeId>11233</BrowseNodeId> 
;;       <Name>Anthropology</Name> 
;;     </BrowseNode>
;;     <BrowseNode>
;;       <BrowseNodeId>11242</BrowseNodeId> 
;;       <Name>Archaeology</Name> 
;;     </BrowseNode>
;;     <BrowseNode>
;;       <BrowseNodeId>3048861</BrowseNodeId> 
;;       <Name>Children's Studies</Name> 
;;     </BrowseNode>
;;   </Children>
;; </BrowseNodes>

;; Limitation: the request only returns one parent browse node for each node.

;;; Finding a Node to Start Your Search
;; - Many high level browse nodes are listed in the Browse Nodes Values section of the A2S API Reference.
;; - The BrowseNodes response group returns browse nodes.
;;   You can use this response group with the following operations:
;;   - ItemSearch
;;   - ItemLookup
;;   - ListLookup
;;   - SimilarityLookup

;; example:
;; http://ecs.amazonaws.com/onca/xml?
;; Service=AWSECommerceService&amp;
;; AWSAccessKeyId=[AWS Access Key ID]&amp;
;; Operation=ItemSearch&amp;
;; Keywords=horse,bridle&amp;
;; SearchIndex=PetSupplies,SportingGoods

;;; Understanding BrowseNode Results When Drilling Down

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Finding Items to Buy > Finding Items on Lists
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; - Each list that a customer creates has a list ID, which is a positive integer.
;; - List IDs are unique by list type.
;; - Finding the items on a customer's list is typically a two-step process:
;;   1. Find the customer's list ID by supplying that customer's name or e-mail address in the ListSearch operations.
;;   2. Use the list ID you find in the ListLookup operation to find the items on the customer's lists

;;; Finding the Customer's List ID

;; example:
;; --
;; http://ecs.amazonaws.com/onca/xml?
;;   Service=AWSECommerceService&
;;   AWSAccessKeyId=[Access Key ID]&
;;   Operation=ListSearch&
;;   ListType=WishList&
;;   Name=John%20Smith
;; --
;; <List>
;;   <ListId>BB4444444477</ListId>
;;   <ListURL>http://www.amazon.com/gp/registry/BB4444444477</ListURL>
;;   <ListType>WishList</ListType>
;;   <TotalItems>70</TotalItems>
;;   <TotalPages>7</TotalPages>
;;   <DateCreated>1999-10-06</DateCreated>
;;   <CustomerName>John Smith</CustomerName>
;; </List>

;; The response contains
;;   - list ID
;;   - list type
;;   - date the list was created
;;   - the URL of the list
;;   - the number of items on the list

;;; Finding the Items Using the Customer ID

;; ListLookup:
;;   - can use many response groups
;;   - requires a ListType
;;   - requires a ListId

;; example:
;; --
;; http://ecs.amazonaws.com/onca/xml?Service=AWSECommerceService&
;; AWSAccessKeyId=[Access Key ID]
;; Operation=ListLookup&
;; ListType=WishList&
;; ListId=BB4444444477
;; --
;; <ListItem>
;;   <ListItemId>98989898989898</ListItemId> 
;;   <DateAdded>2002-01-27</DateAdded> 
;;   <QuantityDesired>1</QuantityDesired> 
;;   <QuantityReceived>0</QuantityReceived> 
;;   <Item>
;;    <ASIN>0865874069</ASIN> 
;;    <ItemAttributes>
;;      <Title>Confined Space Entry : Complying with the Standard</Title> 
;;    </ItemAttributes>
;;   </Item>
;; </ListItem>

;; Notice that the response includes the ListItemId element.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Finding Items to Buy > Finding Customer Information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; CustomerContentSearch

;; example:
;; --
;; http://ecs.amazonaws.com/onca/xml?
;; Service=AWSECommerceService&
;; AWSAccessKeyId=[AWS Access Key ID]&
;; Operation=CustomerContentSearch&
;; Name=Joe%20Schmo
;; --
;; <Customer>
;;   <CustomerId>A3EXAMPLEX53</CustomerId> 
;;   <Nickname>SmokinJoe</Nickname> 
;; </Customer>

;;; CustomerContentLookup

;; The response groups include:
;; - CustomerInfo--(Default) Customer's name, city, and state
;; - CustomerReviews--Product reviews the customer has written
;; - CustomerLists--List IDs of the lists created by the customer.
;; - CustomerFull--All of the above.

;; example:
;; --
;; http://ecs.amazonaws.com/onca/xml?
;; Service=AWSECommerceService&
;; AWSAccessKeyId=[AWS Access Key ID]&
;; Operation=CustomerContentLookup&
;; CustomerId=A2KEKKJ9CAC2KC&
;; ResponseGroup=CustomerLists
;; --
;; <Customer>
;;   <CustomerId>A2EXAMPLEKC</CustomerId> 
;;   <List>
;;     <ListId>BR4444442H77</ListId>
;;     <ListURL>http://www.amazon.com/gp/registry/BR4444442H77
;;       </ListURL>
;;     <ListType>WishList</ListType>
;;     <TotalItems>70</TotalItems>
;;     <TotalPages>7</TotalPages>
;;     <DateCreated>1999-10-06</DateCreated>
;;     <CustomerName>Joe Schmo</CustomerName>
;;   </List>

;; The amount of information that can be returned is restricted by
;; what the customer chose to make publicly accessible.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Locale Considerations > Locales
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define *aws-locales* '(ca de fr jp uk us))

(define aws-locale (make-parameter 'us))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Locale Considerations > General Differences
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; - Items for sale.
;; - Customer feedback.
;; - Sellers and merchants.
;; - Customer accounts are restricted to a locale. A customer can create accounts in every locale.
;; - Item identifiers, such as ASINs, are unique to a locale, that is, the same ASIN value can refer to different items in different locales.
;; - The availability of some response groups, sort parameters, and search indices varies by locale.

(define-class <aws-rest-query> ()
  ((params :init-keyword :params)
   (status :init-keyword :init)
   (headers :init-keyword :headers)
   (body :init-keyword :body)))

(define-method query->uri ((query <aws-rest-query>))
  (uri-compose :scheme "http" :authority (aws-endpoint-host-name)
               :path *aws-endpoint-path-prefix*
               :query (aws-rest-query-string (slot-ref query 'params))))

(define-method execute ((query <aws-rest-query>))
  (sys-sleep 1) ;; a trivial hack
  (receive (status headers body)
      (http-get (aws-endpoint-host-name)
                (string-append *aws-endpoint-path-prefix*
                               "?"
                               (aws-rest-query-string (slot-ref query 'params))))
    (slot-set! query 'status status)
    (slot-set! query 'headers headers)
    (slot-set! query 'body body)
    status))

(define (aws-rest-query-string params)
  (let ((params (acons 'aws-access-key-id (aws-access-key-id)
                       (append *aws-rest-fixed-parameters*
                               (filter cdr params)))))
    (string-join
     (map (lambda (pair)
            (string-append
             (hash-table-get *aws-term-table* (x->string (car pair)))
             "="
             (uri-encode-string (x->string (cdr pair)))))
          params)
     "&")))

(define-syntax define-aws-operation
  (syntax-rules (aws-operation)
    ((_ name
        ((p0 p1 ...) q ...)
        (optional-parameter ...)
        rest ...)
     (define-aws-operation name (q ...) (p0 p1 ... optional-parameter ...) rest ...))
    ((_ name
        (required-parameter ...)
        (optional-parameter ...)
        (response-parameter ...)
        rest ...)
     (begin
       (define (name . args)
         (let-keywords args ((required-parameter #f)
                             ...
                             (optional-parameter #f)
                             ...)
           (if (and required-parameter ...)
               (make <aws-rest-query> :params `((operation . ,(camelize 'name))
                                                     (required-parameter . ,required-parameter) ...
                                                     (optional-parameter . ,optional-parameter) ...))
               (error "some of required parameters are missing:" '(required-parameter ...))
               )))
       (export-with-prefix-aws name)))
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A2S API Reference > Operations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-syntax define-aws-term-table
  (syntax-rules ()
    ((_ (k . v) ...)
     (begin
       (define *aws-term-table* (make-hash-table 'string=?))
       (hash-table-put! *aws-term-table* (x->string 'k) (x->string 'v))
       ...))))

(define-aws-term-table
  (aws-access-key-id . AWSAccessKeyId)
  (service . Service)
  (item-id . ItemId)
  (id-type . IdType)
  (search-index . SearchIndex)

  (about . About)
  (about-me . AboutMe)
  (actor . Actor)
  (additional-name . AdditionalName)
  (address1 . Address1)
  (address2 . Address2)
  (address3 . Address3)
  (all . All)
  (alternate-version . AlternateVersion)
  (amazon-maximum-age . AmazonMaximumAge)
  (amazon-minimum-age . AmazonMinimumAge)
  (amount . Amount)
  (aperture-modes . ApertureModes)
  (apparel . Apparel)
  (artist . Artist)
  (asin . ASIN)
  (aspect-ratio . AspectRatio)
  (audience-rating . AudienceRating)
  (audio-format . AudioFormat)
  (author . Author)
  (automotive . Automotive)
  (availability . Availability)
  (availability-attributes . AvailabilityAttributes)
  (average-feedback-rating . AverageFeedbackRating)
  (average-rating . AverageRating)
  (baby . Baby)
  (back-finding . BackFinding)
  (band-material-type . BandMaterialType)
  (batteries . Batteries)
  (batteries-included . BatteriesIncluded)
  (battery-description . BatteryDescription)
  (battery-type . BatteryType)
  (beauty . Beauty)
  (benefit . Benefit)
  (benefit-description . BenefitDescription)
  (benefit-type . BenefitType)
  (benefits . Benefits)
  (bezel-material-type . BezelMaterialType)
  (bin-item-count . BinItemCount)
  (bin-name . BinName)
  (binding . Binding)
  (birthday . Birthday)
  (blended . Blended)
  (books . Books)
  (brand . Brand)
  (browse-node . BrowseNode)
  (browse-node-id . BrowseNodeId)
  (calendar-type . CalendarType)
  (camera-manual-features . CameraManualFeatures)
  (cart-id . CartId)
  (cart-item . CartItem)
  (cart-item-id . CartItemId)
  (cart-items . CartItems)
  (case-diameter . CaseDiameter)
  (case-material-type . CaseMaterialType)
  (case-thickness . CaseThickness)
  (case-type . CaseType)
  (category . Category)
  (cdrw-description . CDRWDescription)
  (chain-type . ChainType)
  (city . City)
  (clasp-type . ClaspType)
  (classical . Classical)
  (clothing-size . ClothingSize)
  (code . Code)
  (collection . Collection)
  (collection-item . CollectionItem)
  (collection-parent . CollectionParent)
  (collections . Collections)
  (color . Color)
  (comment . Comment)
  (compatibility . Compatibility)
  (component-type . ComponentType)
  (composer . Composer)
  (condition . Condition)
  (condition-note . ConditionNote)
  (conductor . Conductor)
  (content . Content)
  (corrected-query . CorrectedQuery)
  (count . Count)
  (country . Country)
  (coupon-combination-type . CouponCombinationType)
  (cpu-manufacturer . CPUManufacturer)
  (cpu-speed . CPUSpeed)
  (cpu-type . CPUType)
  (creation-date . CreationDate)
  (creator . Creator)
  (currency-amount . CurrencyAmount)
  (currency-code . CurrencyCode)
  (customer-id . CustomerId)
  (customer-name . CustomerName)
  (customer-nickname . CustomerNickname)
  (customer-review . CustomerReview)
  (customer-reviews . CustomerReviews)
  (date . Date)
  (date-added . DateAdded)
  (date-created . DateCreated)
  (day . Day)
  (delay-between-shots . DelayBetweenShots)
  (delivery-method . DeliveryMethod)
  (department . Department)
  (detail-page-url . DetailPageURL)
  (details . Details)
  (dewey-decimal-number . DeweyDecimalNumber)
  (dial-color . DialColor)
  (dial-window-material-type . DialWindowMaterialType)
  (digital-music . DigitalMusic)
  (digital-zoom . DigitalZoom)
  (digitalmusic . digitalmusic)
  (director . Director)
  (display-size . DisplaySize)
  (distinct-users . DistinctUsers)
  (dvd . DVD)
  (dvd-layers . DVDLayers)
  (dvd-sides . DVDSides)
  (dvdrw-description . DVDRWDescription)
  (ean . EAN)
  (edition . Edition)
  (editorial-review-is-link-suppressed . EditorialReviewIsLinkSuppressed)
  (electronics . Electronics)
  (element . Element)
  (eligibility-requirement . EligibilityRequirement)
  (eligibility-requirement-description . EligibilityRequirementDescription)
  (eligibility-requirement-type . EligibilityRequirementType)
  (eligibility-requirements . EligibilityRequirements)
  (end-date . EndDate)
  (esrb-age-rating . ESRBAgeRating)
  (exchange-id . ExchangeId)
  (external-display-support-description . ExternalDisplaySupportDescription)
  (fabric-type . FabricType)
  (fax-number . FaxNumber)
  (feature . Feature)
  (feedback . Feedback)
  (feedback-rating . FeedbackRating)
  (first-issue-lead-time . FirstIssueLeadTime)
  (first-tagging . FirstTagging)
  (fixed-amount . FixedAmount)
  (flavor-name . FlavorName)
  (floppy-disk-drive-description . FloppyDiskDriveDescription)
  (foreign-books . ForeignBooks)
  (foreignbooks . foreignbooks)
  (format . Format)
  (formatted-price . FormattedPrice)
  (gem-type . GemType)
  (gem-type-set-element . GemTypeSetElement)
  (glance-page . GlancePage)
  (golf-club-flex . GolfClubFlex)
  (golf-club-loft . GolfClubLoft)
  (gourmet-food . GourmetFood)
  (gourmetfood . gourmetfood)
  (graphics-card-interface . GraphicsCardInterface)
  (graphics-description . GraphicsDescription)
  (graphics-memory-size . GraphicsMemorySize)
  (grocery . Grocery)
  (group-claim-code . GroupClaimCode)
  (hand-orientation . HandOrientation)
  (hard-disk-count . HardDiskCount)
  (hard-disk-size . HardDiskSize)
  (hardware-platform . HardwarePlatform)
  (has-auto-focus . HasAutoFocus)
  (has-burst-mode . HasBurstMode)
  (has-in-camera-editing . HasInCameraEditing)
  (has-red-eye-reduction . HasRedEyeReduction)
  (has-self-timer . HasSelfTimer)
  (has-tripod-mount . HasTripodMount)
  (has-video-out . HasVideoOut)
  (has-viewfinder . HasViewfinder)
  (health-personal-care . HealthPersonalCare)
  (healthpersonalcare . healthpersonalcare)
  (height . Height)
  (helpful-votes . HelpfulVotes)
  (hmac . HMAC)
  (hobbies . Hobbies)
  (home-garden . HomeGarden)
  (homegarden . homegarden)
  (hours . Hours)
  (hours-of-operation . HoursOfOperation)
  (image . Image)
  (included-software . IncludedSoftware)
  (includes-mp3-player . IncludesMp3Player)
  (industrial . Industrial)
  (ingredients . Ingredients)
  (ingredients-set-element . IngredientsSetElement)
  (is-autographed . IsAutographed)
  (is-category-root . IsCategoryRoot)
  (is-eligible-for-super-saver-shipping . IsEligibleForSuperSaverShipping)
  (is-email-notify-available . IsEmailNotifyAvailable)
  (is-fragile . IsFragile)
  (is-in-benefit-set . IsInBenefitSet)
  (is-in-eligibility-requirement-set . IsInEligibilityRequirementSet)
  (is-lab-created . IsLabCreated)
  (is-link-suppressed . IsLinkSuppressed)
  (is-memorabilia . IsMemorabilia)
  (is-valid . IsValid)
  (isbn . ISBN)
  (iso-equivalent . ISOEquivalent)
  (ispu-postal-code . ISPUPostalCode)
  (issues-per-year . IssuesPerYear)
  (item . Item)
  (item-applicability . ItemApplicability)
  (item-page . ItemPage)
  (jewelry . Jewelry)
  (keyboard-description . KeyboardDescription)
  (keywords . Keywords)
  (kindle-store . KindleStore)
  (kitchen . Kitchen)
  (label . Label)
  (large-image . LargeImage)
  (last-modified . LastModified)
  (last-tagging . LastTagging)
  (legal-disclaimer . LegalDisclaimer)
  (length . Length)
  (line-voltage . LineVoltage)
  (list-id . ListId)
  (list-item-id . ListItemId)
  (list-name . ListName)
  (list-owner . ListOwner)
  (list-type . ListType)
  (list-url . ListURL)
  (listing-id . ListingId)
  (location . Location)
  (loyalty-points . LoyaltyPoints)
  (macro-focus-range . MacroFocusRange)
  (magazine-type . MagazineType)
  (magazines . Magazines)
  (manufacturer . Manufacturer)
  (manufacturer-labor-warranty-description . ManufacturerLaborWarrantyDescription)
  (manufacturer-maximum-age . ManufacturerMaximumAge)
  (manufacturer-minimum-age . ManufacturerMinimumAge)
  (manufacturer-parts-warranty-description . ManufacturerPartsWarrantyDescription)
  (material-type . MaterialType)
  (material-type-set-element . MaterialTypeSetElement)
  (maximum-aperture . MaximumAperture)
  (maximum-color-depth . MaximumColorDepth)
  (maximum-focal-length . MaximumFocalLength)
  (maximum-high-resolution-images . MaximumHighResolutionImages)
  (maximum-horizontal-resolution . MaximumHorizontalResolution)
  (maximum-hours . MaximumHours)
  (maximum-low-resolution-images . MaximumLowResolutionImages)
  (maximum-price . MaximumPrice)
  (maximum-resolution . MaximumResolution)
  (maximum-shutter-speed . MaximumShutterSpeed)
  (maximum-vertical-resolution . MaximumVerticalResolution)
  (maximum-weight-recommendation . MaximumWeightRecommendation)
  (medium-image . MediumImage)
  (memory-slots-available . MemorySlotsAvailable)
  (merchandising-message . MerchandisingMessage)
  (merchant-id . MerchantId)
  (merchants . Merchants)
  (message . Message)
  (metal-stamp . MetalStamp)
  (metal-type . MetalType)
  (mini-movie-description . MiniMovieDescription)
  (minimum-focal-length . MinimumFocalLength)
  (minimum-hours . MinimumHours)
  (minimum-price . MinimumPrice)
  (minimum-shutter-speed . MinimumShutterSpeed)
  (miscellaneous . Miscellaneous)
  (model . Model)
  (modem-description . ModemDescription)
  (monitor-size . MonitorSize)
  (monitor-viewable-diagonal-size . MonitorViewableDiagonalSize)
  (mouse-description . MouseDescription)
  (mp3-downloads . MP3Downloads)
  (mp3downloads . mp3downloads)
  (mpaa-rating . MPAARating)
  (mpn . MPN)
  (music . Music)
  (music-label . MusicLabel)
  (music-tracks . MusicTracks)
  (musical-instruments . MusicalInstruments)
  (musicalinstruments . musicalinstruments)
  (musictracks . musictracks)
  (name . Name)
  (native-resolution . NativeResolution)
  (neighborhood . Neighborhood)
  (network-interface-description . NetworkInterfaceDescription)
  (nickname . Nickname)
  (notebook-display-technology . NotebookDisplayTechnology)
  (notebook-pointing-device-description . NotebookPointingDeviceDescription)
  (number . Number)
  (number-of-discs . NumberOfDiscs)
  (number-of-issues . NumberOfIssues)
  (number-of-items . NumberOfItems)
  (number-of-pages . NumberOfPages)
  (number-of-pearls . NumberOfPearls)
  (number-of-rapid-fire-shots . NumberOfRapidFireShots)
  (number-of-stones . NumberOfStones)
  (number-of-tracks . NumberOfTracks)
  (occasion-date . OccasionDate)
  (offer-listing-id . OfferListingId)
  (office-products . OfficeProducts)
  (officeproducts . officeproducts)
  (operating-system . OperatingSystem)
  (operation . Operation)
  (optical-zoom . OpticalZoom)
  (orchestra . Orchestra)
  (original-release-date . OriginalReleaseDate)
  (other-categories-similar-products . OtherCategoriesSimilarProducts)
  (outdoor-living . OutdoorLiving)
  (outdoorliving . outdoorliving)
  (owning-merchant-id . OwningMerchantId)
  (package-quantity . PackageQuantity)
  (parameter . Parameter)
  (parent-asin . ParentASIN)
  (partner-name . PartnerName)
  (pc-hardware . PCHardware)
  (pchardware . pchardware)
  (pearl-lustre . PearlLustre)
  (pearl-minimum-color . PearlMinimumColor)
  (pearl-shape . PearlShape)
  (pearl-stringing-method . PearlStringingMethod)
  (pearl-surface-blemishes . PearlSurfaceBlemishes)
  (pearl-type . PearlType)
  (pearl-uniformity . PearlUniformity)
  (performer . Performer)
  (pet-supplies . PetSupplies)
  (petsupplies . petsupplies)
  (phone-number . PhoneNumber)
  (photo . Photo)
  (photo-flash-type . PhotoFlashType)
  (picture-format . PictureFormat)
  (platform . Platform)
  (postal-code . PostalCode)
  (power . Power)
  (price . Price)
  (price-rating . PriceRating)
  (processor-count . ProcessorCount)
  (product-group . ProductGroup)
  (product-type-subcategory . ProductTypeSubcategory)
  (promotion . Promotion)
  (promotion-category . PromotionCategory)
  (promotion-id . PromotionId)
  (promotions . Promotions)
  (publication-date . PublicationDate)
  (publisher . Publisher)
  (purchase-url . PurchaseURL)
  (quantity . Quantity)
  (quantity-desired . QuantityDesired)
  (quantity-received . QuantityReceived)
  (rating . Rating)
  (reading-level . ReadingLevel)
  (region-code . RegionCode)
  (registry-name . RegistryName)
  (registry-number . RegistryNumber)
  (release-date . ReleaseDate)
  (removable-memory . RemovableMemory)
  (request-id . RequestId)
  (resolution-modes . ResolutionModes)
  (response-group . ResponseGroup)
  (ring-size . RingSize)
  (role . Role)
  (running-time . RunningTime)
  (sales-rank . SalesRank)
  (saved-for-later-item . SavedForLaterItem)
  (scent-name . ScentName)
  (search-bin-set . SearchBinSet)
  (search-bin-sets . SearchBinSets)
  (secondary-cache-size . SecondaryCacheSize)
  (seller-id . SellerId)
  (seller-legal-name . SellerLegalName)
  (seller-nickname . SellerNickname)
  (setting-type . SettingType)
  (shaft-material-type . ShaftMaterialType)
  (shipment-items . ShipmentItems)
  (shipments . Shipments)
  (silver-merchant . SilverMerchant)
  (silver-merchants . SilverMerchants)
  (similar-products . SimilarProducts)
  (similar-viewed-products . SimilarViewedProducts)
  (single-merchant-id . SingleMerchantId)
  (size . Size)
  (size-per-pearl . SizePerPearl)
  (sku . SKU)
  (small-image . SmallImage)
  (software . Software)
  (software-video-games . SoftwareVideoGames)
  (softwarevideogames . softwarevideogames)
  (sort . Sort)
  (sound-card-description . SoundCardDescription)
  (source . Source)
  (speaker-description . SpeakerDescription)
  (special-features . SpecialFeatures)
  (sporting-goods . SportingGoods)
  (sportinggoods . sportinggoods)
  (sportingoods . sportingoods)
  (start-date . StartDate)
  (state . State)
  (status . Status)
  (stone-clarity . StoneClarity)
  (stone-color . StoneColor)
  (stone-cut . StoneCut)
  (stone-shape . StoneShape)
  (stone-weight . StoneWeight)
  (store-id . StoreId)
  (store-name . StoreName)
  (studio . Studio)
  (style . Style)
  (sub-condition . SubCondition)
  (subject . Subject)
  (subscription-length . SubscriptionLength)
  (summary . Summary)
  (supported-image-type . SupportedImageType)
  (swatch-image . SwatchImage)
  (system-bus-speed . SystemBusSpeed)
  (system-memory-size . SystemMemorySize)
  (system-memory-size-max . SystemMemorySizeMax)
  (system-memory-type . SystemMemoryType)
  (tag . Tag)
  (tag-name . TagName)
  (tag-type . TagType)
  (tagged-items . TaggedItems)
  (tags . Tags)
  (terms-and-conditions . TermsAndConditions)
  (text-stream . TextStream)
  (theatrical-release-date . TheatricalReleaseDate)
  (thumbnail-image . ThumbnailImage)
  (time . Time)
  (tiny-image . TinyImage)
  (title . Title)
  (tools . Tools)
  (total-collectible . TotalCollectible)
  (total-diamond-weight . TotalDiamondWeight)
  (total-external-bays-free . TotalExternalBaysFree)
  (total-feedback . TotalFeedback)
  (total-feedback-pages . TotalFeedbackPages)
  (total-firewire-ports . TotalFirewirePorts)
  (total-gem-weight . TotalGemWeight)
  (total-internal-bays-free . TotalInternalBaysFree)
  (total-items . TotalItems)
  (total-metal-weight . TotalMetalWeight)
  (total-new . TotalNew)
  (total-ntscpal-ports . TotalNTSCPALPorts)
  (total-offer-pages . TotalOfferPages)
  (total-offers . TotalOffers)
  (total-pages . TotalPages)
  (total-parallel-ports . TotalParallelPorts)
  (total-pc-card-slots . TotalPCCardSlots)
  (total-pci-slots-free . TotalPCISlotsFree)
  (total-ratings . TotalRatings)
  (total-refurbished . TotalRefurbished)
  (total-results . TotalResults)
  (total-review-pages . TotalReviewPages)
  (total-reviews . TotalReviews)
  (total-s-video-out-ports . TotalSVideoOutPorts)
  (total-serial-ports . TotalSerialPorts)
  (total-times-read . TotalTimesRead)
  (total-usages . TotalUsages)
  (total-usb-ports . TotalUSBPorts)
  (total-usb2-ports . TotalUSB2Ports)
  (total-used . TotalUsed)
  (total-vga-out-ports . TotalVGAOutPorts)
  (total-votes . TotalVotes)
  (totals . Totals)
  (toys . Toys)
  (track . Track)
  (transaction-date . TransactionDate)
  (transaction-date-epoch . TransactionDateEpoch)
  (transaction-id . TransactionId)
  (transaction-item . TransactionItem)
  (transaction-item-id . TransactionItemId)
  (transaction-items . TransactionItems)
  (type . Type)
  (unit . Unit)
  (upc . UPC)
  (url . URL)
  (url-encoded-hmac . URLEncodedHMAC)
  (user-agent . UserAgent)
  (user-id . UserId)
  (value . Value)
  (variation-denomination . VariationDenomination)
  (variation-description . VariationDescription)
  (variation-dimension . VariationDimension)
  (version . Version)
  (vhs . VHS)
  (video . Video)
  (video-games . VideoGames)
  (videogames . videogames)
  (warranty . Warranty)
  (watch-movement-type . WatchMovementType)
  (watches . Watches)
  (water-resistance-depth . WaterResistanceDepth)
  (weight . Weight)
  (width . Width)
  (will-ship-expedited . WillShipExpedited)
  (will-ship-international . WillShipInternational)
  (wireless . Wireless)
  (wireless-accessories . WirelessAccessories)
  (wish-list-id . WishListId)
  )

(define-aws-operation browse-node-lookup
  (browse-node-id)
  (response-group)
  (ancestors browse-node browse-node-id children name)
  :availability #t
  )

(define-aws-operation cart-add
  ((asin offer-listing-id) cart-id hmac quantity)
  (list-item-id merge-cart response-group)
  (amount asin cart-add-request cart-id currency-code formatted-price hmac item items purchase-url quantity sub-total)
  :availability #t
  )

(define-aws-operation cart-clear
  (cart-id hmac)
  (associate-tag merge-cart response-group)
  (cart cart-clear-request cart-id hmac url-encoded-hmac)
  :availability #t
  )

(define-aws-operation cart-create
  ((asin offer-listing-id) item)
  (associate-tag list-item-id merge-cart quantity response-group)
  (amount asin cart cart-create-request cart-id cart-items currency-code formatted-price hmac item-total purchase-url quantity saved-for-later-items sub-total url-encoded-hmac)
  :availability #t
  )

(define-aws-operation cart-get
  (cart-id cart-item-id hmac)
  (associate-tag merge-cart response-group)
  (amount cart-id cart-item cart-items currency-code formatted-price hmac item-total price purchase-url sub-total url-encoded-hmac)
  :availability #t
  )

(define-aws-operation cart-modify
  (cart-id cart-item-id hmac quantity)
  (action list-item-id merge-cart response-group)
  (cart-id cart-item-id hmac item items quantity)
  :availability #t
  )

(define-aws-operation customer-content-lookup
  (customer-id)
  (review-page tag-page tags-per-page tag-sort response-group)
  (customer customers nickname tags)
  :availability '(us de fr jp uk)
  )

(define-aws-operation customer-content-search
  ((e-mail name) customer-page)
  (response-group)
  (city country customer customers location nickname state)
  :availability '(us)
  )

(define-aws-operation help
  ()
  (about help-type response-group)
  (available-parameters available-response-groups default-response-group element elements name operation operation-information required-parameters valid-operations)
  :availability #t
  )

(define-aws-operation item-lookup
  (item-id)
  (condition id-type merchant-id offer-page review-page review-sort search-index tag-page tags-per-page tag-sort variation-page response-group)
  (asin item item-attributes items manufacturer product-group title)
  :availability #t ; NOTE: the parameter support varies by locale though.
  )

(define-aws-operation item-search
  ((availability condition merchant-id)) ; DG says just 'yes' for required, while its detailed description are opposite.
  (actor artist audience-rating author brand browse-node city composer conductor director item-page keywords manufacturer maximum-price minimum-price neighborhood orchestra postal-code power publisher search-index sort tag-page tags-per-page tag-sort text-stream title variation-page response-group)
  (asin detail-page-url item item-attributes manufacturer product-group title total-pages total-results)
  :availability #t
  )

(define-aws-operation list-lookup
  (list-id list-type)
  (condition merchant-id product-group product-page sort response-group)
  (customer-name date-created list-name list-type total-items total-pages)
  :availability #t
  )

(define-aws-operation list-search
  ()
  (city content-type e-mail first-name last-name list-page name state response-group)
  (customer-name date-created list list-id lists list-url total-items total-pages total-results)
  :availability #t
  )

(define-aws-operation seller-listing-lookup
  (id id-type)
  (seller-id response-group)
  (amount availability currency-code end-date exchange-id featured-category formatted-price listing-id nick-name price quantity quantity-allocated seller seller-id start-date status title)
  :availability #t
  )

(define-aws-operation seller-listing-search
  (seller-id)
  (listing-page offer-status sort title response-group)
  (amount availability currency-code end-date exchange-id featured-category formatted-price listing-id nick-name price quantity quantity-allocated seller seller-id start-date status title)
  ;;:availability #t
  )

(define-aws-operation seller-lookup
  (seller-id)
  (feedback-page response-group)
  (average-feedback-rating city comment date feedback glance-page location nick-name rating seller-id state total-feedback total-feedback-pages)
  ;;:availability #t
  )

(define-aws-operation similarity-lookup
  (item-id)
  (condition merchant-id similarity-type response-group)
  (asin detail-page-url item item-attributes response-group title)
  ;;:availability #t
  )

(define-aws-operation tag-lookup
  (tag-name)
  (count customer-id tag-page tag-sort response-group)
  (asin customer-id distinct-users first-tagging item last-tagging name tag tagged-items tag-name tags tag-type time total-usages user-id)
  :availability '(us)
  )

(define-aws-operation transaction-lookup
  (transaction-id)
  (response-group)
  (amount condition currency-code formatted-price promotion quantity seller-id seller-name shipment shipment-items shipping-charge subtotal total totals transaction-date transaction-date-epoch transaction-id transaction-item transaction-item-id transaction-items unit-price)
  :availability '(us)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A2S API Reference > Common Request Parameters
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-syntax define-aws-common-request-parameter
  (syntax-rules ()
    ((_ (r ...) (o ...))
     '((:required r ...)
       (:optional o ...)))))
       
(define-aws-common-request-parameter
  (aws-access-key-id operation service)
  (associate-tag content-type merchant-id style validate version xml-escaping))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A2S API Reference > Request Groups
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-syntax define-aws-request-group
  (syntax-rules ()
    ((_ name (operation ...) (element ...) rest ...)
     `(name (operation ...) (element ...) rest ...))
    ((_ name x y rest ...)
     '(name x y rest ...))
    ))

(define-aws-request-group accessories
  (similarity-lookup item-lookup item-search list-lookup)
  (asin title total-pages total-results)
  :parent-response-groups (large)
  )

(define-aws-request-group alternate-versions
  (item-search item-lookup)
  (alternate-version asin binding title)
  )

(define-aws-request-group browse-node-info
  (browse-node-lookup)
  (browse-node-id name)
  )

(define-aws-request-group browse-nodes
  (item-lookup item-search list-lookup similarity-lookup)
  (browse-node-id is-category-root name total-pages total-results)
  )

(define-aws-request-group cart
  (cart-add cart-create cart-modify cart-get cart-clear)
  (amount asin cart-id cart-item cart-item-id cart-items currency-code exchange-id formatted-price hmac list-owner list-type list-type merchant-id parent-asin price product-group product-group purchase-url quantity saved-for-later-item seller-id seller-nickname title url-encoded-hmac)
  )

(define-aws-request-group cart-new-releases
  (cart-add cart-create cart-modify cart-get)
  (asin title)
  )

(define-aws-request-group cart-top-sellers
  (cart-add cart-create cart-modify cart-get)
  (asin title)
  )

(define-aws-request-group cart-similarities
  (cart-add cart-create cart-modify cart-get)
  (asin other-categories-similar-products similar-products similar-viewed-products title)
  )

(define-aws-request-group collections
  (item-lookup item-search)
  (asin collection collection-item collection-parent collections title)
  )

(define-aws-request-group customer-full
  (customer-content-lookup)
  (about-me asin birthday city content country customer-id customer-reviews date helpful-votes nickname rating state summary total-votes total-review-pages total-reviews total-pages total-results wish-list-id)
  :child-response-groups (customer-info customer-lists customer-reviews)
  )

(define-aws-request-group customer-info
  (customer-content-lookup customer-content-search)
  (customer-id city country nickname state total-pages total-results)
  )

(define-aws-request-group customer-lists
  (customer-content-lookup)
  (customer-id wish-list-id)
  )

(define-aws-request-group customer-reviews
  (customer-content-lookup)
  (asin customer-reviews content date helpful-votes rating summary total-votes total-review-pages total-reviews)
  )

(define-aws-request-group editorial-reviews
  (item-lookup item-search list-lookup similarity-lookup)
  (content editorial-review-is-link-suppressed source)
  )

(define-aws-request-group help
  (help)
  (creation-date element name operation parameter response-group version)
  )

(define-aws-request-group images
  (item-lookup item-search list-lookup similarity-lookup)
  (height large-image medium-image small-image swatch-image thumbnail-image tiny-image total-pages total-results url width)
  )

(define-aws-request-group item-attributes
  (item-lookup item-search list-lookup similarity-lookup)
  (actor address1 address2 address3 amazon-maximum-age amazon-minimum-age amount aperture-modes artist asin aspect-ratio audience-rating audio-format author back-finding band-material-type batteries batteries-included battery-description battery-type bezel-material-type binding brand calendar-type camera-manual-features case-diameter case-material-type case-thickness case-type cdrw-description chain-type city clasp-type clothing-size color compatibility cpu-manufacturer cpu-speed cpu-type creator currency-code day delay-between-shots department detail-page-url dewey-decimal-number dial-color dial-window-material-type digital-zoom director display-size dvd-layers dvdrw-description dvd-sides ean edition esrb-age-rating external-display-support-description fabric-type fax-number feature first-issue-lead-time flavor-name floppy-disk-drive-description format formatted-price gem-type gem-type-set-element golf-club-flex golf-club-loft graphics-card-interface graphics-description graphics-memory-size hard-disk-count hard-disk-size has-auto-focus has-burst-mode has-in-camera-editing has-red-eye-reduction has-self-timer has-tripod-mount has-video-out has-viewfinder height hours hours-of-operation included-software includes-mp3-player ingredients ingredients-set-element is-autographed isbn isemail-notify-available is-fragile is-lab-created is-memorabilia iso-equivalent issues-per-year keyboard-description keywords label legal-disclaimer length line-voltage macro-focus-range magazine-type manufacturer manufacturer-labor-warranty-description manufacturer-maximum-age material-type-set-element manufacturer-minimum-age manufacturer-parts-warranty-description material-type maximum-aperture maximum-color-depth maximum-focal-length maximum-high-resolution-images maximum-horizontal-resolution maximum-low-resolution-images maximum-resolution maximum-shutter-speed maximum-vertical-resolution maximum-weight-recommendation memory-slots-available message metal-stamp metal-type mini-movie-description minimum-focal-length minimum-shutter-speed model modem-description monitor-size monitor-viewable-diagonal-size mouse-description mpn name native-resolution neighborhood network-interface-description notebook-display-technology notebook-pointing-device-description number-of-discs number-of-issues number-of-items number-of-pages number-of-pearls number-of-rapid-fire-shots number-of-stones number-of-tracks optical-zoom original-release-date pearl-lustre pearl-minimum-color pearl-shape pearl-stringing-method pearl-surface-blemishes pearl-type pearl-uniformity phone-number photo-flash-type picture-format platform postal-code price-rating processor-count product-group publication-date publisher reading-level region-code release-date removable-memory resolution-modes ring-size role running-time secondary-cache-size setting-type size size-per-pearl sku sound-card-description speaker-description special-features state stone-clarity stone-color stone-cut stone-shape stone-weight studio subscription-length supported-image-type system-bus-speed system-memory-size system-memory-size-max system-memory-type theatrical-release-date title total-diamond-weight total-external-bays-free total-firewire-ports total-gem-weight total-internal-bays-free total-metal-weight total-ntscpal-ports total-pages total-parallel-ports total-pc-card-slots total-pci-slots-free total-results total-serial-ports total-s-video-out-ports total-usb-ports total-usb2-ports total-vga-out-ports type unit upc variation-denomination variation-description warranty watch-movement-type water-resistance-depth weight width)
  )

(define-aws-request-group item-ids
  (item-lookup item-search list-lookup similarity-lookup)
  (asin corrected-query keywords message total-pages total-results)
  )

(define-aws-request-group large
  (item-lookup item-search list-lookup similarity-lookup)
  (actor address1 address2 address3 amazon-maximum-age amazon-minimum-age amount aperture-modes artist asin aspect-ratio audience-rating audio-format author back-finding band-material-type batteries batteries-included battery-description battery-type bezel-material-type binding brand browse-node-id calendar-type camera-manual-features case-diameter case-material-type case-thickness case-type cdrw-description chain-type city clasp-type clothing-size color compatibility content cpu-manufacturer cpu-speed cpu-type creator currency-code day delay-between-shots department detail-page-url dewey-decimal-number dial-color dial-window-material-type digital-zoom director display-size dvd-layers dvdrw-description dvd-sides ean edition esrb-age-rating external-display-support-description fabric-type fax-number feature first-issue-lead-time flavor-name floppy-disk-drive-description format formatted-price gem-type golf-club-flex golf-club-loft graphics-card-interface graphics-description graphics-memory-size hard-disk-count hard-disk-size has-auto-focus has-burst-mode has-in-camera-editing has-red-eye-reduction has-self-timer has-tripod-mount has-video-out has-viewfinder height hours hours-of-operation included-software includes-mp3-player ingredients is-autographed isbn is-fragile is-lab-created is-memorabilia iso-equivalent issues-per-year keyboard-description keywords label legal-disclaimer length line-voltage list-id list-name macro-focus-range magazine-type manufacturer manufacturer-labor-warranty-description manufacturer-maximum-age manufacturer-minimum-age manufacturer-parts-warranty-description material-type maximum-aperture maximum-color-depth maximum-focal-length maximum-high-resolution-images maximum-horizontal-resolution maximum-low-resolution-images maximum-resolution maximum-shutter-speed maximum-vertical-resolution maximum-weight-recommendation memory-slots-available message metal-stamp metal-type mini-movie-description minimum-focal-length minimum-shutter-speed model modem-description monitor-size monitor-viewable-diagonal-size mouse-description mpn name native-resolution neighborhood network-interface-description notebook-display-technology notebook-pointing-device-description number-of-discs number-of-issues number-of-items number-of-pages number-of-pearls number-of-rapid-fire-shots number-of-stones number-of-tracks optical-zoom pearl-lustre pearl-minimum-color pearl-shape pearl-stringing-method pearl-surface-blemishes pearl-type pearl-uniformity phone-number photo-flash-type picture-format platform postal-code price-rating processor-count product-group publication-date publisher reading-level region-code release-date removable-memory resolution-modes ring-size role running-time sales-rank secondary-cache-size setting-type size size-per-pearl sku sound-card-description source speaker-description special-features state stone-clarity stone-color stone-cut stone-shape stone-weight studio subscription-length supported-image-type system-bus-speed system-memory-size system-memory-size-max system-memory-type theatrical-release-date title total-collectible total-diamond-weight total-external-bays-free total-firewire-ports total-gem-weight total-internal-bays-free total-used total-metal-weight total-new total-ntscpal-ports total-pages total-parallel-ports total-pc-card-slots total-pci-slots-free total-refurbished total-results total-serial-ports total-s-video-out-ports total-usb2-ports total-usb-ports total-vga-out-ports type unit upc url variation-denomination variation-description warranty watch-movement-type water-resistance-depth weight width)
  :child-response-groups (accessories browse-nodes listmania-lists medium offers reviews similarities tracks)
  )

(define-aws-request-group list-full
  (list-lookup)
  (additional-name asin average-rating comment comment customer-name customer-nickname date-added date-created image last-modified list-id list-item-id list-name list-type list-url occasion-date partner-name quantity-desired quantity-received registry-name registry-number title total-items total-pages total-results total-times-read total-votes)
  :child-response-groups (list-info list-items)
  )

(define-aws-request-group list-info
  (list-lookup list-search)
  (additional-name average-rating comment customer-name customer-nickname date-created image last-modified list-id list-name list-type list-url occasion-date partner-name registry-name total-items total-pages total-results total-times-read)
  :parent-response-groups (list-full)
  :child-response-groups (list-minimum)
  )

(define-aws-request-group list-items
  (list-lookup)
  (asin comment date-added list-id list-item-id list-name quantity-desired quantity-received title total-pages total-results)
  :child-response-groups (list-minimum)
  )

(define-aws-request-group listmania-lists
  (item-lookup item-search list-lookup similarity-lookup)
  (list-id list-name)
  )

(define-aws-request-group list-minimum
  (list-search)
  (list-id list-name total-items total-pages)
  )

(define-aws-request-group midium
  (item-lookup item-search list-lookup similarity-lookup)
  (actor address1 address2 address3 amazon-maximum-age amazon-minimum-age amount aperture-modes artist asin aspect-ratio audience-rating audio-format author back-finding band-material-type batteries batteries-included battery-description battery-type bezel-material-type binding brand calendar-type camera-manual-features case-diameter case-material-type case-thickness case-type cdrw-description chain-type city clasp-type clothing-size color compatibility content cpu-manufacturer cpu-speed cpu-type creator currency-code day delay-between-shots department detail-page-url dewey-decimal-number dial-color dial-window-material-type digital-zoom director display-size dvd-layers dvdrw-description dvd-sides ean edition esrb-age-rating external-display-support-description fabric-type fax-number feature first-issue-lead-time flavor-name floppy-disk-drive-description format formatted-price gem-type golf-club-flex golf-club-loft graphics-card-interface graphics-description graphics-memory-size hard-disk-count hard-disk-size has-auto-focus has-burst-mode has-in-camera-editing has-red-eye-reduction has-self-timer has-tripod-mount has-video-out has-viewfinder height hours hours-of-operation included-software includes-mp3-player ingredients is-autographed isbn is-fragile is-lab-created is-memorabilia iso-equivalent issues-per-year keyboard-description keywords label legal-disclaimer length line-voltage macro-focus-range magazine-type manufacturer manufacturer-labor-warranty-description manufacturer-maximum-age manufacturer-minimum-age manufacturer-parts-warranty-description material-type maximum-aperture maximum-color-depth maximum-focal-length maximum-high-resolution-images maximum-horizontal-resolution maximum-low-resolution-images maximum-resolution maximum-shutter-speed maximum-vertical-resolution maximum-weight-recommendation memory-slots-available message metal-stamp metal-type mini-movie-description minimum-focal-length minimum-shutter-speed model modem-description monitor-size monitor-viewable-diagonal-size mouse-description mpn name name native-resolution neighborhood network-interface-description notebook-display-technology notebook-pointing-device-description number-of-discs number-of-issues number-of-items number-of-pages number-of-pearls number-of-rapid-fire-shots number-of-stones number-of-tracks optical-zoom pearl-lustre pearl-minimum-color pearl-shape pearl-stringing-method pearl-surface-blemishes pearl-type pearl-uniformity phone-number photo-flash-type picture-format platform postal-code price-rating processor-count product-group publication-date publisher reading-level region-code release-date removable-memory resolution-modes ring-size role running-time sales-rank secondary-cache-size setting-type size size-per-pearl sku sound-card-description source speaker-description special-features state stone-clarity stone-color stone-cut stone-shape stone-weight studio subscription-length supported-image-type system-bus-speed system-memory-size system-memory-size-max system-memory-type theatrical-release-date title total-collectible total-diamond-weight total-external-bays-free total-firewire-ports total-gem-weight total-internal-bays-free total-used total-metal-weight total-new total-ntscpal-ports total-pages total-parallel-ports total-pc-card-slots total-pci-slots-free total-refurbished total-results total-serial-ports total-s-video-out-ports total-usb2-ports total-usb-ports total-vga-out-ports type unit upc url variation-denomination variation-description warranty watch-movement-type water-resistance-depth weight width)
  :child-response-groups (editorial-review images item-attributes offer-summary request sales-rank small)
  )

(define-aws-request-group merchant-item-attributes
  (item-lookup item-search)
  item-attributes ; same as those described for ItemAttributes response group
  )

(define-aws-request-group new-releases
  (browse-node-lookup)
  (asin title)
  )

(define-aws-request-group offer-full
  (item-lookup item-search list-lookup similarity-lookup)
  (address1 address2 address3 amount availability average-feedback-rating city condition condition-note country country currency-code exchange-id formatted-price glance-page is-eligible-for-super-saver-shipping merchant-id name nickname offer-listing-id postal-code quantity seller-id state sub-condition total-collectible total-feedback total-new total-offer-pages total-offers total-refurbished total-used)
  :child-response-groups (offers)
  )

(define-aws-request-group offer-listings
  (item-lookup item-search list-lookup similarity-lookup)
  (amount availability average-feedback-rating code condition condition-note country currency-code exchange-id formatted-price glance-page is-eligible-for-super-saver-shipping merchant-id message name offer-listing-id seller-id state sub-condition total-feedback total-offer-pages total-offers value)
  )

(define-aws-request-group offers
  (item-lookup item-search list-lookup similarity-lookup)
  (amount availability average-feedback-rating condition condition-note currency-code exchange-id formatted-price glance-page is-eligible-for-super-saver-shipping loyalty-points merchant-id name offer-listing-id quantity postal-code seller-id state sub-condition total-collectible total-feedback total-new total-offer-pages total-offers total-refurbished total-used)
  :child-response-groups (offer-summary)
  )

(define-aws-request-group offer-summary
  (item-lookup item-search list-lookup similarity-lookup)
  (amount currency-code formatted-price total-collectible total-new total-refurbished total-used)
  )

(define-aws-request-group promotion-details
  (item-lookup item-search list-lookup similarity-lookup)
  (amount asin benefit benefits benefit-type component-type coupon-combination-type currency-amount currency-code details eligibility-requirement eligibility-requirement-type eligibility-requirements end-date fixed-amount formatted-price group-claim-code is-in-benefit-set is-in-eligibility-requirement-set item-applicability merchandising-message merchant-id owning-merchant-id promotion promotion-category promotion-id promotions start-date terms-and-conditions)
  )

(define-aws-request-group promotion-summary
  (item-lookup item-search list-lookup similarity-lookup)
  (benefit-description category eligibility-requirement-description end-date promotion promotion-id promotions start-date summary terms-and-conditions)
  )

(define-aws-request-group request
  #t ; All A2S operations use the Request response group by default
  (code is-valid message name request-id user-agent value)
  )

(define-aws-request-group reviews
  (item-lookup item-search list-lookup similarity-lookup)
  (asin average-rating content customer-id date helpful-votes rating summary total-pages total-results total-review-pages total-reviews total-votes)
  )

(define-aws-request-group sales-rank
  (item-lookup item-search list-lookup similarity-lookup)
  (asin sales-rank total-pages total-results)
  )

(define-aws-request-group search-bins
  (item-search)
  (bin-item-count bin-name name search-bin-sets value)
  :availability (us)
  )

(define-aws-request-group seller
  (seller-lookup)
  (about feedback feedback-rating location name seller-legal-name seller-nickname total-feedback total-feedback-pages total-ratings)
  )

(define-aws-request-group seller-listing
  (seller-listing-lookup seller-listing-search)
  (amount asin condition currency-code ean end-date exchange-id formatted-price image listing-id nickname quantity seller-id sku start-date status store-id store-name sub-condition title upc will-ship-expedited will-ship-international)
  )

(define-aws-request-group similarities
  (item-lookup item-search list-lookup similarity-lookup)
  (asin title total-pages total-results)
  )

(define-aws-request-group small
  (item-lookup item-search list-lookup similarity-lookup)
  (actor artist asin author corrected-query creator detail-page-url director keywords manufacturer message product-group role title total-pages total-results)
  :parent-response-groups (large medium)
  )

(define-aws-request-group subjects
  (item-lookup item-search list-lookup similarity-lookup)
  (subject)
  )

(define-aws-request-group tagged-guides
  (customer-content-lookup item-lookup item-search tag-lookup)
  (name tag tags tag-type total-usages)
  )

(define-aws-request-group tagged-items
  (customer-content-lookup item-lookup item-search tag-lookup)
  (customer-id distinct-users first-tagging item last-tagging name tag tagged-items tag-name tags tag-type time total-usages user-id)
  )

(define-aws-request-group tagged-listmania-lists
  (customer-content-lookup item-lookup item-search tag-lookup)
  (distinct-users first-tagging item last-tagging name t-tag tags tag-type total-usages user-id)
  )

(define-aws-request-group tags
  (customer-content-lookup item-lookup item-search tag-lookup)
  (asin customer-id distinct-users first-tagging item last-tagging name tag tagged-items tag-name tags tag-type time total-usages user-id)
  :child-response-groups (tagged-guides tagged-items tagged-listmania-lists tags-summary)
  )

(define-aws-request-group tags-summary
  (customer-content-lookup item-lookup item-search tag-lookup)
  (asin item name tag tagged-items tag-name tags total-usages)
  )

(define-aws-request-group top-sellers
  (browse-node-lookup)
  (asin title)
  )

(define-aws-request-group tracks
  (item-lookup item-search list-lookup similarity-lookup)
  (number total-pages total-results track)
  )

(define-aws-request-group transaction-details
  (transaction-lookup)
  (amount currency-code formatted-price shipment-items shipments totals transaction-date transaction-date-epoch transaction-id transaction-item transaction-item-id transaction-items)
  )

(define-aws-request-group variation-minimum
  (item-lookup item-search list-lookup similarity-lookup)
  (asin)
  :parent-response-groups (variations)
  )

(define-aws-request-group variations
  (item-lookup item-search list-lookup similarity-lookup)
  (amount asin currency-code flavor-name formatted-price golf-club-flex golf-club-loft single-merchant-id)
  :child-response-groups (variation-minimum variation-summary)
  )

(define-aws-request-group variation-images
  (item-lookup)
  (height large-image medium-image small-image swatch-image thumbnail-image tiny-image url width)
  )

(define-aws-request-group variation-matrix
  (item-lookup item-search)
  (clothing-size color flavor-name gem-type golf-club-flex golf-club-loft hand-orientation hardware-platform item-dimensions/length item-dimensions/width material-type maximum-weight-recommendation metal-type model operating-system package-quantity product-type-subcategory ring-size scent-name shaft-material-type size style total-diamond-weight total-gem-weight variation-dimension)
  )

(define-aws-request-group variation-offers
  (item-lookup item-search)
  (amount asin availability availability-attributes average-feedback-rating condition condition-note currency-code exchange-id formatted-price glance-page is-eligible-for-super-saver-shipping loyalty-points maximum-hours merchant-id minimum-hours name offer-listing-id quantity postal-code seller-id state sub-condition total-collectible total-feedback total-new total-offer-pages total-offers)
  :parent-response-groups (variations)
  )

(define-aws-request-group variation-summary
  (item-lookup item-search list-lookup similarity-lookup)
  (amount currency-code formatted-price single-merchant-id)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A2S API Reference > Common Response Elements
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-syntax define-aws-common-response-elements
  (syntax-rules ()
    ((_ (name (ancestry ...)) ...)
     `((name . (ancestry ... name)) ...))))

(define-aws-common-response-elements
  (asin (item))
  (code (errors error))
  (is-valid (request))
  (message (errors error))
  (name (arguments argument))
  (request-id (operation-request))
  (total-pages (list))
  (total-results (list))
  (user-agent (operation-request))
  (value (arguments argument))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A2S API Reference > All Response Elements
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-syntax define-aws-response-elements
  (syntax-rules ()
    ((_ (element description ...) ...)
     '((element description ...) ...))))

(define-aws-response-elements
  (about
   "Describes the seller. "
   "Ancestry:  Seller/About "
   )
  (about-me
   "Information a customer supplies about themselves "
   "Ancestry:  Seller/About "
   )
  (actor
   "Actor associated with the item."
   "Ancestry:  ItemAttributes"
   )
  (additional-name
   "For baby registries, this field is used for the name of the other parent. "
   "Ancestry:  List/AdditionalName"
   )
  (address1
   "Street address "
   "Ancestry:  ItemAttributes/Address "
   "Offers/Offer/OfferListing/Address1"
   )
  (address2
   "Street address "
   "Ancestry:  ItemAttributes/Address"
   "Offers/Offer/OfferListing/Address2"
   )
  (address3
   "Street address "
   "Ancestry:  ItemAttributes/Address"
   "Offers/Offer/OfferListing/Address3"
   )
  (alternate-version
   "Container for AlternateVersion information, including ASIN, Title, and Binding."
   "Ancestry:  None"
   )
  (amazon-maximum-age
   "Maximum age in months of the person who will enjoy the item. For example, a book might be targeted at kids up to three years old. Three would be the value for the MaximumAge."
   "Ancestry:  ItemAttributes"
   )
  (amazon-minimum-age
   "Youngest age in months that a person should be to use the item. "
   "Ancestry:  ItemAttributes"
   )
  (amount
   "Price in terms of the lowest currency denomination, for example, pennies. The price, in terms of the lowest currency denomination, for example, pennies, of an item in the cart. The total price, in terms of the lowest currency denomination, of one or more of the same item in the Saved For Later area. If, for example, Saved For Later contains two orders of the same book, this amount would be the sum total of those two books."
   "Ancestry:  ItemAttributes/ListPrice OfferSummary/LowestUsedPrice"
   "OfferSummary/LowestCollectiblePrice "
   "OfferSummary/LowestNewPrice OfferSummary/LowestRefurbishedPrice/Amount  "
   "Offers/Offer/OfferListing/Price/Amount"
   "Offers/Offer/OfferListing/SalePrice/Amount"
   "Transaction/TransactionItems/TransactionItem/UnitPrice "
   "VariationSummary/LowestPrice"
   "VariationSummary/HighestSalePrice "
   )
  (aperture-modes
   "Aperture modes associated with items that have lenses."
   "Ancestry:  ItemAttributes"
   )
  (artist
   "Artist associated with the item."
   "Ancestry:  ItemAttributes"
   )
  (asin
   "A positive integer distributed by Amazon that uniquely identifies an item. ASIN of the new release, item, or similar item"
   "Ancestry: Item/ASIN CartItem SavedForLaterItem Cart/NewReleases/NewRelease NewReleases/NewRelease/ASIN OtherCategoriesSimilarProduct/ASIN SimilarProduct SimilarViewedProduct, CustomerReviews/Review "
   )
  (aspect-ratio
   "The ratio of an item's length to its width."
   "Ancestry:  ItemAttributes"
   )
  (audience-rating
   "Audience rating for a movie. The rating suggests the age for which the movie is appropriate. The rating format varies by locale. "
   "Ancestry: ItemAttributes "
   )
  (audio-format
   "Format, such as MP3, of the audio media. "
   "Ancestry:  ItemAttributes/Languages/Language"
   )
  (author
   "Author associated with the item."
   "Ancestry:  ItemAttributes"
   )
  (availability
   "How soon the item can be shipped. "
   "Ancestry:  Offers/Offer/OfferListing/Availability"
   )
  (availability-attributes
   "Container for availability information, including AvailabilityType, MaximumHours and MinimumHours. "
   "Ancestry:  Offers/Offer/OfferListing"
   )
  (average-feedback-rating
   "Customer review of an item represented by a number of stars where 5 stars is the highest rating, 0 the lowest. This element contains the average of all customer ratings of the item. "
   "Ancestry:  Offers/Offer/Seller/AverageFeedbackRating"
   )
  (average-rating
   "An average of all of the ratings entered by customers where 5 is the best. "
   "Ancestry: CustomerReviews, List "
   )
  (back-finding
   "The metal in jewelry, for example, 14 K gold. "
   "Ancestry: ItemAttributes "
   )
  (band-material-type
   "Type of material used to make the band in a piece of jewelry. "
   "Ancestry:  ItemAttributes"
   )
  (batteries
   "Batteries required or not "
   "Ancestry:  ItemAttributes"
   )
  (batteries-included
   "Whether batteries are included "
   "Ancestry:  ItemAttributes"
   )
  (battery-description
   "A description of the batteries that are required for the item to work."
   "Ancestry:  ItemAttributes"
   )
  (battery-type
   "Required battery type "
   "Ancestry:  ItemAttributes"
   )
  (benefit
   "Container for information relating to a promotional benefit. "
   "Ancestry:  Benefits"
   )
  (benefits
   "Containter for one or more Benefit elements. "
   "Ancestry:  None"
   )
  (benefit-type
   "Specifies the type of promotion, for example, Free, FixedAmountOff, PercentOff, and TieredFixedAmountOff. "
   "Ancestry:  Benefits/Benefit/BenefitType"
   )
  (benefit-description
   "Text that describes the promotion. "
   "Ancestry:  Offers/Offer/Promotions/Promotion/Summary"
   )
  (bezel-material-type
   "Material that makes up the bezel of jewelry, for example, a ring. "
   "Ancestry:  ItemAttributes"
   )
  (bin-item-count
   "Number of items in a bin. "
   "Ancestry: SearchBinSets/SearchBinSet/Bin/BinItemCount  "
   )
  (bin-name
   "Name of the bin. "
   "Ancestry:  SearchBinSets/SearchBinSet/Bin/BinName"
   )
  (binding
   "Typically but not always similar to the product category."
   "Ancestry:  ItemAttributes"
   )
  (birthday
   "A customer's birth date. "
   "Ancestry:  CustomerFull"
   )
  (brand
   "An item's brand. "
   "Ancestry:  ItemAttributes, Large, Medium"
   )
  (browse-node-id
   "A positive integer that uniquely identifies a parent product category. "
   "Ancestry:  BrowseNode/Ancestors/BrowseNode BrowseNode/Children/BrowseNode"
   )
  (calendar-type
   "Type of calendar "
   "Ancestry:  ItemAttributes"
   )
  (camera-manual-features
   "Manual features of a camera "
   "Ancestry:  ItemAttributes"
   )
  (cart-id
   "A positive integer that uniquely identifies a cart. All operations on a cart must include this value, which is generated by CartCreate. "
   "Ancestry:  Cart"
   )
  (cart-item
   "A parent element for many child elements, including CartItemId, MerchantId, SellerId, Quantity, Title, ProductGroup, Price, and ItemTotal."
   "Ancestry:  Cart/CartItems"
   )
  (cart-item-id
   "A positive integer that uniquely identifies an item in a cart or in the Saved For Later area. You must use this value to modify cart items or Saved For Later items. Other identifiers, such as ASINs, do not work. "
   "Ancestry:  CartItem"
   "SavedForLaterItem"
   )
  (cart-items
   "A parent element for many child elements, including SubTotal, and CartItem."
   "Ancestry:  Cart"
   )
  (case-diameter
   "Distance from corner to corner of a case "
   "Ancestry:  ItemAttributes"
   )
  (case-material-type
   "The material of which a case is made. "
   "Ancestry:  ItemAttributes"
   )
  (case-thickness
   "Thickness of a case "
   "Ancestry:  ItemAttributes"
   )
  (case-type
   "Type of case "
   "Ancestry:  ItemAttributes"
   )
  (category
   "Specifies the kind of promotion. Valid values include FreeShipping, BuyAmountXGetAmountOffX, and ForEachAmountXGetAmountOffX. For more information, see "
   ". "
   "Ancestry:  Offers/Offer/Promotions/Promotion/Summary"
   "Promotion Types"
   )
  (cdrw-description
   "Description of CD read-write peripheral device. "
   "Ancestry:  ItemAttributes"
   )
  (chain-type
   "Type of chain, typically, in jewelry. "
   "Ancestry:  ItemAttributes"
   )
  (city
   "City "
   "Ancestry:  ItemAttributes/Address"
   )
  (clasp-type
   "Type of clasp on a piece of jewelry "
   "Ancestry:  ItemAttributes"
   )
  (clothing-size
   "Size of clothes "
   "Ancestry:  ItemAttributes"
   )
  (code
   "Number that uniquely identifies an error."
   "Ancestry:  Errors/Error"
   )
  (collection
   "Container for items that are part of a collection. For more information, see "
   "Ancestry:  None"
   "Collections"
   )
  (collection-item
   "An item that is part of a collection of items. For more information, see "
   "Ancestry:  None"
   "Collections"
   )
  (collection-parent
   " The parent item has an ASIN but it cannot be purchase. It names the collection. CollectionItems are children of the Collection Parent and can be purchased. For more information, see "
   "Ancestry:  None"
   "Collections"
   )
  (collections
   "Container for one or more collection objects. For more information, see "
   "Ancestry:  None"
   "Collections"
   )
  (color
   "Color "
   "Ancestry:  ItemAttributes"
   )
  (comment
   "Comment typically about the purpose of the list."
   "Ancestry:  List, ListItems"
   )
  (compatibility
   "Compatibility "
   "Ancestry:  ItemAttributes"
   )
  (component-type
   "Specifies what the promotion applies to, for example, Shipping, ItemPrice, Subtotal. "
   "Ancestry:  Benefits/Benefit"
   )
  (condition
   "Specifies the condition of the item, such as new, used, collectible, or refurbished. "
   "Ancestry:  SellerListing/Condition Offers/Offer/OfferAttributes/Condition"
   )
  (condition-note
   "A note that describes the Condition. "
   "Ancestry:  "
   )
  (content
   "The content of the review. "
   "Ancestry:  Offers/Offer/OfferAttributes/ConditionNote"
   )
  (corrected-query
   "A parent element that contains the elements related to a corrected keyword. See Keywords. "
   "Ancestry:  Items"
   )
  (country
   "A customer's country. "
   "Ancestry:  CustomerFull, CustomerInfo, OfferFull, OfferListings"
   )
  (coupon-combination-type
   "Specifies the kinds of promotional coupons that can be combined, for example, Unrestricted, Preferential, and Exclusive. "
   "Ancestry: None "
   )
  (cpu-manufacturer
   "Manufacturer of the CPU "
   "Ancestry:  ItemAttributes"
   )
  (cpu-speed
   "CPU speed "
   "Ancestry:  ItemAttributes"
   )
  (cpu-type
   "CPU type, such as P4. "
   "Ancestry:  ItemAttributes"
   )
  (creator
   "Creator associated with the item. "
   "Ancestry:  ItemAttributes"
   )
  (creation-date
   "The date an API element was created."
   "Ancestry:  Help"
   )
  (currency-amount
   "Price in terms of the lowest currency denomination, for example, pennies. "
   "Ancestry:  EligibilityRequirements/EligibilityRequirement/"
   )
  (currency-code
   "An abbreviation that specifies the format of the price for the associated locale. "
   "Ancestry:"
   "  ItemAttributes/ListPrice "
   "CartItem/ItemTotal "
   "Cart/SavedForLaterItems/SubTotal"
   "Cart/CartItems/SubTotal"
   "SavedForLaterItem/ItemTotal"
   "SavedForLaterItem/Price "
   "CartItem/Price "
   "Cart/SubTotal "
   "EligibilityRequirements/EligibilityRequirement/CurrencyAmount"
   "Benefits/Benefit/FixedAmount "
   "OfferSummary/LowestCollectiblePrice/CurrencyCode"
   "OfferSummary/LowestNewPrice/CurrencyCode"
   "OfferSummary/LowestRefurbishedPrice/CurrencyCode"
   "OfferSummary/LowestUsedPrice/CurrencyCode"
   "Offers/Offer/OfferListing/Price/CurrencyCode"
   "Offers/Offer/OfferListing/SalePrice/CurrencyCode"
   "Transaction/Totals/Total "
   "VariationSummary/HighestPrice\n                 VariationSummary/LowestSalePrice\n                 \n                 VariationSummary/HighestSalePrice\n                 \n                 VariationSummary/LowestPrice"
   )
  (customer-id
   "A positive number distributed by Amazon that unqiely identifies a customer who entered a review. "
   "Ancestry:  CustomerReviews/Review/CustomerId"
   )
  (customer-name
   "Name of the list owner. "
   "Ancestry:  List/CustomerName"
   )
  (customer-nickname
   "Nickname of list owner "
   "Ancestry:  List"
   )
  (customer-review
   "A review of a seller submitted by a customer."
   "Ancestry:  CustomerFull, CustomerReviews"
   )
  (customer-reviews
   "A containter for one or more CustomerReview objects."
   "Ancestry:  CustomerFull, CustomerReviews"
   )
  (date
   "The date the review was created. "
   "Ancestry:  CustomerReviews/Review"
   )
  (date-added
   "Date the item was added to the list. "
   "Ancestry:  ListItem/DateAdded"
   )
  (date-created
   "Date the list was created, in the form yyyy-mm-dd. "
   "Ancestry:  List/DateCreated"
   )
  (day
   "Day "
   "Ancestry:  ItemAttributes/HoursOfOperation/Hours "
   )
  (delay-between-shots
   "The necessary delay between shots. "
   "Ancestry:  ItemAttributes"
   )
  (department
   "Department "
   "Ancestry: ItemAttributes "
   )
  (details
   "Container for all of the elements that describe a promotion. "
   "Ancestry:  "
   )
  (detail-page-url
   "URL of the detail page, which describes an item and enables the customer to add it to their cart. "
   "Ancestry:  ItemAttributes"
   )
  (dewey-decimal-number
   "Dewey decimal number "
   "Ancestry:  ItemAttributes"
   )
  (dial-color
   "Color of the dial, typically, of a watch face "
   "Ancestry: ItemAttributes "
   )
  (dial-window-material-type
   "Material that makes up the transparent face of a watch. "
   "Ancestry:  ItemAttributes"
   )
  (digital-zoom
   "Specifies the zoom ratio. "
   "Ancestry:  ItemAttributes"
   )
  (director
   "Director "
   "Ancestry:  ItemAttributes"
   )
  (display-size
   "Size of the display, typically the length of the diagonal across the display surface. "
   "Ancestry:  ItemAttributes"
   )
  (distinct-users
   "Number of different users that created tags for an entity."
   "Ancestry:  Tagged Items, TaggedListmaniaLists, Tag"
   )
  (dvd-layers
   "Specifies whether the DVD is a dual-layer DVD-9 movie or a single-layer DVD-5 movie. "
   "Ancestry:  ItemAttributes"
   )
  (dvdrw-description
   "Description of the DVD player's read and write functionality. "
   "Ancestry:  ItemAttributes"
   )
  (dvd-sides
   "Specifies whether one or two sides of the DVD has recorded information. "
   "Ancestry:  ItemAttributes"
   )
  (ean
   "European Article Number, which is a number that uniquely identifies an item. "
   "Ancestry:  ItemAttributes"
   )
  (edition
   "Edition "
   "Ancestry:  ItemAttributes"
   )
  (editorial-review-is-link-suppressed
   "Boolean value that specifies whether or not the link to the editorial review is displayed. "
   "Ancestry:  EditorialReview"
   )
  (element
      "An xml tag returned. "
    "Ancestry:  Help"
    )
  (eligibility-requirement
   "Containter for one or more EligibilityRequirementType elements. "
   "Ancestry:  EligibilityRequirements"
   )
  (eligibility-requirement-description
   "Specifies the conditions necessary to qualify for the promotion. "
   "Ancestry:  Offers/Offer/Promotions/Promotion/Summary"
   )
  (eligibility-requirements
   "Containter for one or more EligibilityRequirement elements. "
   "Ancestry:  None"
   )
  (eligibility-requirement-type
   "Specifies the eligibility requirements to qualify for the promotion, for example, MinPurchase, MinQuantity, or, ForEachQuantity. "
   "Ancestry:  EligibilityRequirements/EligibilityRequirement"
   )
  (end-date
   "Specifies a date when the item will stop being sold. Or, specifies the last day and ending time (GMT) of the promotion. "
   "Ancestry:  SellerListing/EndDate, Offers/Offer/Promotions/Promotion/Summary"
   )
  (exchange-id
   "Token that uniquely identifies an SDP-Marketplace, auction, Merchant@,  Merchant.Com listings, or an offer in the Marketplace. Exchange IDs are available for every item offered by a third-party seller. When using ExchangeId, you must also specify a matching SellerID and ASIN."
   "Ancestry:  "
   "SellerListing"
   "SavedForLaterItem"
   "OfferSummary/LowestNewPrice/FormattedPrice"
   "OfferSummary/LowestRefurbishedPrice/FormattedPrice"
   "OfferSummary/LowestUsedPrice/FormattedPrice"
   "Offers/Offer/OfferListing/Price/FormattedPrice"
   "Offers/Offer/OfferListing/SalePrice/FormattedPrice"
   "CartItem"
   )
  (esrb-age-rating
   "The Entertainment Software Rating Board (ESRB) sets ratings, advertising guidelines, and online privacy principles for video and computer games in the United States and Canada. The value, CEROAgeRating specifies that a video or software game (search indices Software and VideoGames) is suitable for everyone to play. "
   "Ancestry:  ItemAttributes"
   )
  (external-display-support-description
   "Specifies whether the items supports an external display. "
   "Ancestry:  ItemAttributes"
   )
  (fabric-type
   "The type of fabric used in the item. "
   "Ancestry:  ItemAttributes"
   )
  (fax-number
   "Fax number "
   "Ancestry: ItemAttributes "
   )
  (feature
   "An item's feature "
   "Ancestry:  ItemAttributes"
   )
  (feedback
   "Provides customer feedback about the seller. "
   "Ancestry:  "
   )
  (feedback-rating
   "Provides an average customer rating of the merchant where 5 is the best and 0 is the worst. "
   "Ancestry:  Seller/FeedbackRating"
   )
  (first-issue-lead-time
   "The wait time to expect before receiving the first issue in a newspaper or magazine subscription. "
   "Ancestry:  ItemAttributes"
   )
  (first-tagging
   "Container for information relating to the first time the entity was tagged."
   "Ancestry:  TaggedItems, TaggedListmaniaLists, Tag"
   )
  (fixed-amount
   "Specifies the price in terms of the lowest currency denomination. "
   "Ancestry:  Benefits/Benefit"
   )
  (flavor-name
   "The flavor of an item. Typically used as a variation dimension."
   "Ancestry:  ItemAttributes"
   )
  (floppy-disk-drive-description
   "Description of a computer's floopy drive. "
   "Ancestry:  ItemAttributes"
   )
  (format
   "An item's format "
   "Ancestry:  ItemAttributes"
   )
  (formatted-price
   "The price formatted as it should be displayed to the user. The formatting is specified by CurrencyCode and varies by country. Or, the discounted price formatted for display."
   "Ancestry: ItemAttributes/ListPrice "
   "SavedForLaterItem/ExchangeId "
   "CartItem/ExchangeId, "
   "EligibilityRequirements/EligibilityRequirement/CurrencyAmount"
   "Benefits/Benefit/FixedAmount "
   "OfferSummary/LowestUsedPrice/FormattedPrice "
   "OfferSummary/LowestCollectiblePrice/FormattedPrice "
   "OfferSummary/LowestNewPrice/FormattedPrice "
   "Transaction/TransactionItems/TransactionItem/TotalPrice"
   )
  (gem-type
   "The type of gem. "
   "Ancestry:  ItemAttributes/GemType"
   )
  (gem-type-set-element
   "Returns multiple values for the gem types."
   "Ancestry:  Variations/Item/ItemAttributes"
   )
  (glance-page
   "URL of a seller's web page. A GlancePage is not returned when the merchant is Amazon."
   "Ancestry:  "
   "Offers/Offer/Seller/Glancepage"
   "Offers/Offer/Merchant/GlancePage"
   )
  (golf-club-flex
   "The flex (regular or stiff) of a golf club."
   "Ancestry:  ItemAttributes"
   )
  (golf-club-loft
   "The loft (number of degrees) of a golf club."
   "Ancestry:  ItemAttributes"
   )
  (graphics-card-interface
   "The type of computer graphic card interface. "
   "Ancestry: ItemAttributes "
   )
  (graphics-description
   "Description of the computer graphics card. "
   "Ancestry:  ItemAttributes"
   )
  (graphics-memory-size
   "The amount of memory resident on in a computer graphics card. "
   "Ancestry:  ItemAttributes"
   )
  (group-claim-code
   "An alphanumeric token to use to claim the promotional benefit. "
   "Ancestry:  None"
   )
  (hand-orientation
   "Left or right handedness"
   "Ancestry:  Variations/Item/ItemAttributes"
   )
  (hard-disk-count
   "Number of hard drives in a computer. "
   "Ancestry:  ItemAttributes"
   )
  (hard-disk-size
   "The amount of memory contained in a hard drive. "
   "Ancestry:  ItemAttributes"
   )
  (hardware-platform
   "Hardware"
   "Ancestry:  Variations/Item/ItemAttributes"
   )
  (has-auto-focus
   "Specifies whether the item has auto focus. "
   "Ancestry:  ItemAttributes"
   )
  (has-burst-mode
   "Specifies whether or not the item has burst mode. "
   "Ancestry:  ItemAttributes"
   )
  (has-in-camera-editing
   "Specifies whether a camera has editing functionality. "
   "Ancestry:  ItemAttributes"
   )
  (has-red-eye-reduction
   "Specifies whether or not the camera has a mechanism for preventing red eyes in pictures. "
   "Ancestry:  ItemAttributes"
   )
  (has-self-timer
   "Specifies whether or not the item has a timer. "
   "Ancestry:  ItemAttributes"
   )
  (has-tripod-mount
   "Specifies whether or not the item has a mount to connect to a tripod. "
   "Ancestry:  ItemAttributes"
   )
  (has-video-out
   "Specifies whether or not the item has an output for video. "
   "Ancestry:  ItemAttributes"
   )
  (has-viewfinder
   "Specifies whether or not the item has a view finder. "
   "Ancestry:  ItemAttributes"
   )
  (height
   "Height of an item, package, or image."
   "Ancestry:  ItemAttributes/ItemDimensions ItemAttributes/PackageDimensions ImageSets/ImageSet/LargeImage ImageSets/ImageSet/MediumImage ImageSets/ImageSet/SmallImage ImageSets/ImageSet/TinyImaget ImageSets/ImageSet/SwatchImage/Height ItemAttributes"
   )
  (helpful-votes
   "The number of customers who felt that the review was helpful. "
   "Ancestry:  CustomerReviews/Review/HelpfulVotes"
   )
  (hmac
   "Hashing for Message Authentication (HMAC) is a keyed hash function that is used with a CryptographicHashFunction such as SHA-1 or MD5 that cannot be computed with out the key. This is used to identify and secure a cart. Each cart operation must include this value."
   "Ancestry: Cart "
   )
  (hours
   "Hours "
   "Ancestry:  ItemAttributes/HoursOfOperation"
   )
  (hours-of-operation
   "Defines the hours of a store's operations. "
   "Ancestry:  ItemAttributes"
   )
  (image
   "An image associated with a seller's item. "
   "Ancestry:  SellerListing/Image"
   )
  (included-software
   "Specifies the software included in the purchase. "
   "Ancestry: ItemAttributes "
   )
  (includes-mp3-player
   "Specifies whether or not the item contains an MP3 player. "
   "Ancestry:  ItemAttributes"
   )
  (ingredients
   "Ingredient"
   "Ancestry:  ItemAttributes"
   )
  (ingredients-set-element
   "Multiple ingredients. Used instead of Ingredients when the item is made of more than one ingredient."
   "Ancestry: Variations/Item/ItemAttributes "
   )
  (is-autographed
   "Specifies whether or not the item is autographed. "
   "Ancestry:  ItemAttributes"
   )
  (isbn
   "ISBN number "
   "Ancestry:  ItemAttributes"
   )
  (is-category-root
   "Boolean value that specifies if the browse node is at the top of the browse node tree. "
   "Ancestry:  BrowseNodes"
   )
  (is-eligible-for-super-saver-shipping
   "Specifies whether the items is eligible for super saving shipping. "
   "Ancestry: Offers/Offer/OfferListing/IsEligibleForSuperSaverShipping "
   )
  (is-email-notify-available
   "The value of IsEmailNotifyAvailable is a boolean where 1 means that an Email can be sent when the item is in stock, 0 when an Email cannot be sent. This functionality is not available for all offers. "
   )
  (is-fragile
   "Species whether or not the item is fragile. "
   "Ancestry:  ItemAttributes"
   )
  (is-in-benefit-set
   "A boolean value. When True, the associated ASIN is what the customer receives as part of the promotion."
   "Ancestry:  ItemApplicability"
   )
  (is-in-eligibility-requirement-set
   "A boolean value. When True, the associated ASIN is what the customer must purchase to qualify for the promotion."
   "Ancestry:  ItemApplicability"
   )
  (is-lab-created
   "Specifies whether or not the item was created in a lab. "
   "Ancestry:  ItemAttributes"
   )
  (is-link-suppressed
   "\"1\" (true) if there were any HTML links in editorial reviews that have been suppressed. "
   "Ancestry:  EditorialReview"
   )
  (is-memorabilia
   "Specifies whether the item is considered memorabilia. "
   "Ancestry:  ItemAttributes"
   )
  (iso-equivalent
   "The ISO equivalent of a digital camera is what the film sensitivity would be if the camera used film. The higher the number, the more sensitive the camera, the less light required to take the photograph."
   "Ancestry:  ItemAttributes"
   )
  (item
   "Container for information about an item."
   "Ancestry:  ItemAttributes"
   )
  (item-applicability
   "Container for ASIN, IsInBenefitSet, and IsInEligibilityRequirementSet. This group of values specifies whether the ASIN is what the customer receives as part of the promotion, whether the ASIN is what the customer needs to purchase to qualify for the promotion, or both. "
   "Ancestry:  "
   "ItemApplicability/ItemApplicability"
   )
  (issues-per-year
   "Number of issues per year, usually, in a subscription. "
   "Ancestry:  ItemAttributes"
   )
  (is-valid
   "Boolean value that specifies whether the request syntax was correct. "
   "Ancestry:  Request"
   )
  (keyboard-description
   "Description of the keyboard "
   "Ancestry:  ItemAttributes"
   )
  (keywords
   "The (corrected) words used in a search. "
   "Ancestry:  "
   "Items/CorrectedQuery "
   )
  (label
   "Label "
   "Ancestry:  ItemAttributes"
   )
  (large-image
   "Container for a large image "
   "Ancestry:  "
   "ImageSets/ImageSet"
   )
  (last-modified
   "Date when the list was last modified "
   "Ancestry:  "
   "List/LastModified "
   )
  (last-tagging
   "Container for information relating to the last time the entity was tagged."
   "Ancestry:  TaggedItems, TaggedListmaniaLists, Tag"
   )
  (legal-disclaimer
   "Legal disclaimer "
   "Ancestry:  ItemAttributes"
   )
  (length
   "Length "
   "Ancestry:  ItemAttributes/ItemDimensions ItemAttributes/PackageDimensions/Length ItemAttributes"
   )
  (line-voltage
   "The voltage used. "
   "Ancestry:  ItemAttributes"
   )
  (list-id
   "A positive integer distributed by Amazon that uniquely identifies a list. "
   "Ancestry:  ListmaniaLists/ListmaniaList "
   )
  (listing-id
   "A number that uniquely identifies a seller's item within a given year. This values, also known as the Exchange Code, is a shortened form of the ExchangeId, whose uniqueness is not limited to a single year. "
   "Ancestry:  SellerListing/ListingId"
   )
  (list-item-id
   "A number that uniquely identifies an item on a list. "
   "Ancestry: ListItem/ListItemId "
   )
  (list-name
   "Name of a Listmania list. "
   "Ancestry:  "
   "ListmaniaLists/ListmaniaList List"
   )
  (list-owner
   "Name of the list owner. "
   "Ancestry:  "
   "CartItem "
   "SavedForLaterItem "
   )
  (list-type
   "The type of list an item in the cart is on. The item might be in the Saved For Later area. "
   "Ancestry:  "
   "CartItem "
   "SavedForLaterItem "
   )
  (list-url
   "URL where you can find the list. The URL is the same for all lists except for the final filename, which is the ListId. In this way you can display the list by just knowing the ListId."
   "Ancestry: List "
   )
  (location
   "Location of the seller. "
   "Ancestry:  "
   "Seller/Location "
   )
  (loyalty-points
   "In the JP locale only, loyalty points are returned. Loyalty points are used to generate sales."
   "Ancestry:  "
   "Offers/Offer"
   )
  (loyalty-points
   "The number of points awarded as part of a purchase. Points translate into rewards."
   "Ancestry:  "
   "Offers, VariationOffers"
   )
  (macro-focus-range
   "Describes the macro focus feature. "
   "Ancestry:  ItemAttributes"
   )
  (magazine-type
   "Describes the genre of the magazine. "
   "Ancestry:  ItemAttributes"
   )
  (manufacturer
   "Manufacturer "
   "Ancestry:  ItemAttributes"
   )
  (manufacturer-labor-warranty-description
   "Describes the manufacturer's labor warranty "
   "Ancestry:  ItemAttributes"
   )
  (manufacturer-maximum-age
   "Defines the maximum age in months the user should be to enjoy the use of the item. For example, for a toy targeted at kids from ages 2 to 4, 4 would be the value for the ManufacturerMaximumAge. "
   "Ancestry:  ItemAttributes"
   )
  (manufacturer-minimum-age
   "Defines the minimum age in months the user should be to enjoy the use of the item. For example, for a toy targeted at kids from ages 2 to 4, 2 would be the value for the ManufacturerMiniimumAge. "
   "Ancestry:  ItemAttributes"
   )
  (manufacturer-parts-warranty-description
   "Describes the manufacturer's parts warranty "
   "Ancestry:  ItemAttributes"
   )
  (material-type
   "The type of material used in the item. "
   "Ancestry:  ItemAttributes"
   )
  (material-type-set-element
   "Returns multiple values describing the different materials that constitute the item."
   "Ancestry:  "
   "Variations/Item/ItemAttributes"
   )
  (maximum-aperture
   "The larges lens aperture "
   "Ancestry:  ItemAttributes"
   )
  (maximum-color-depth
   "The number of bytes used to display colors in digital media. "
   "Ancestry:  ItemAttributes"
   )
  (maximum-focal-length
   "The maximum focal length of the lens. "
   "Ancestry:  ItemAttributes"
   )
  (maximum-high-resolution-images
   "The highest image resolution provided by the item."
   "Ancestry:  ItemAttributes"
   )
  (maximum-hours
   "The maximum number of hours for which the item might be available."
   "Ancestry:  "
   "Offers/Offer/OfferListing/AvailabilityAttributes"
   )
  (maximum-horizontal-resolution
   "The highest resolution provided on the horizontal axis--the higher the number, the greater the resolution. "
   "Ancestry:  None"
   )
  (maximum-low-resolution-images
   "The lowest image resolution an item provides. "
   "Ancestry:  ItemAttributes"
   )
  (maximum-resolution
   "The highest image resolution an item provides. "
   "Ancestry:  ItemAttributes"
   )
  (maximum-shutter-speed
   "The fastest speed of a camera shutter, This value is an of time.The unit of time is captured in the "
   " element. "
   "Ancestry:  ItemAttributes"
   "Units"
   )
  (maximum-vertical-resolution
   "Highest image resolution on the vertical axis. "
   "Ancestry:  ItemAttributes"
   )
  (maximum-weight-recommendation
   "The maximum weight an item should hold. "
   "Ancestry:  ItemAttributes"
   )
  (medium-image
   "Container for a medium image "
   "Ancestry:  "
   "ImageSets/ImageSet"
   )
  (memory-slots-available
   "Specifies whether or not the item has slots for memory additions. "
   "Ancestry:  ItemAttributes"
   )
  (merchandising-message
   "MerchandisingMessage contains text entered by a merchant that describes a promotion."
   "Ancestry:  None"
   )
  (merchant-id
   "A positive integer distributed by Amazon that uniquely identifies a merchant. Or, the legacy merchant ID that is associated with the promotion. In contrast, the element, OwningMerchantId, is an updated ID for the same merchant. These IDs may or may not be the same."
   "Ancestry:  "
   "CartItem "
   "SavedForLaterItem "
   )
  (message
   "Message, or, a description of an error. "
   "Ancestry:  "
   "Items/CorrectedQuery, Errors/Error "
   )
  (metal-stamp
   "A distinctive marking in metal usually associated with jewelry. "
   "Ancestry:  ItemAttributes"
   )
  (metal-type
   "The type of metal used in the item. "
   "Ancestry:  ItemAttributes"
   )
  (minimum-hours
   "The minimum number of hours for which the item is available."
   "Ancestry:  "
   "Offers/Offer/OfferListing/AvailabilityAttributes"
   )
  (mini-movie-description
   "Short description of a movie. "
   "Ancestry:  ItemAttributes"
   )
  (minimum-focal-length
   "The shortest focal length of an item. "
   "Ancestry:  ItemAttributes"
   )
  (minimum-shutter-speed
   "The longest time a camera shutter can stay open. "
   "Ancestry:  ItemAttributes"
   )
  (model
   "Model "
   "Ancestry:  ItemAttributes"
   )
  (modem-description
   "Description of a modem. "
   "Ancestry:  ItemAttributes"
   )
  (monitor-size
   "Size of a monitor expressed as the length between opposite corners of the monitor "
   "Ancestry:  ItemAttributes"
   )
  (monitor-viewable-diagonal-size
   "The length between opposite corners of the monitor in which the image actually appears. "
   "Ancestry:  ItemAttributes"
   )
  (mouse-description
   "Description of a computer mouse. "
   "Ancestry:  ItemAttributes"
   )
  (mpn
   "Manufacturer's part number. "
   "Ancestry:  ItemAttributes"
   )
  (name
   "Name of customer of item in a bin. Or, the name of a parameter passed in the request. "
   "Ancestry:  ItemAttributes/Address ItemAttributes/Languages/Language BrowseNode/Ancestors/BrowseNode BrowseNode/Children/BrowseNode, Seller, SearchBinSets, SearchBinSet/Bin/BinParameter, Arguments/Argument"
   )
  (native-resolution
   "The maximum resolution of a flat panel display. "
   "Ancestry:  ItemAttributes"
   )
  (neighborhood
   "Neighborhood "
   "Ancestry:  ItemAttributes"
   )
  (network-interface-description
   "Description of the item's network interface. "
   "Ancestry: ItemAttributes "
   )
  (nickname
   "Seller's nickname "
   "Ancestry:  "
   "SellerListing/Seller "
   )
  (notebook-display-technology
   "Describes the technology used in the notebook computer's display "
   "Ancestry:  ItemAttributes"
   )
  (notebook-pointing-device-description
   "Describes the pointing device used by the notebook computer. "
   "Ancestry:  ItemAttributes"
   )
  (number
   "An attribute in the Disc tag that denotes the CD number in the item, for example, <Disc Number=\"1\">. Items with multiple CDs would have multiple values for Disc. Or, an attribute in the Track tag that denotes the track number on the CD, for example, <Track Number=\"9\">Now Or Never</Track>."
   "Ancestry:  "
   "Tracks/Disc"
   "Tracks/Disc/Track"
   )
  (number-of-discs
   "Number of discs an item can hold or use. "
   "Ancestry:  ItemAttributes"
   )
  (number-of-issues
   "Number of issues in a subscription. "
   "Ancestry:  ItemAttributes"
   )
  (number-of-items
   "Number of items. "
   "Ancestry:  ItemAttributes"
   )
  (number-of-pages
   "Number of pages. "
   "Ancestry:  ItemAttributes"
   )
  (number-of-pearls
   "Number of pearls in an item "
   "Ancestry:  ItemAttributes"
   )
  (number-of-rapid-fire-shots
   "The number of shots that can be fired sequentially. "
   "Ancestry:  ItemAttributes"
   )
  (number-of-stones
   "Number of stones in an item "
   "Ancestry:  ItemAttributes"
   )
  (number-of-tracks
   "Number of recorded track on a CD. "
   "Ancestry:  ItemAttributes"
   )
  (occasion-date
   "Specifies the date of an occasion, such as a wedding or birthday. "
   "Ancestry:  "
   "List/OccasionDate "
   )
  (offer-listing-id
   "A number that uniquely identifies an offer listing. This number represents a sales offer from a specific merchant. "
   "Ancestry:  "
   "Offers/Offer/OfferListing"
   )
  (operating-system
   "Specifies the name of the operating system on a computer."
   "Ancestry:  VariationMatrix"
   )
  (operation
   "Specifies the name of the operation, such as ItemLookup."
   "Ancestry:  Help"
   )
  (optical-zoom
   "Specifies the optical, as opposed to the electronic, zoom. "
   "Ancestry:  ItemAttributes"
   )
  (original-release-date
   "Specifies the date the item was originally released. Items that have been released multiple times have both release dates and original release dates. See ReleaseDate."
   "Ancestry: ItemAttributes "
   )
  (other-categories-similar-products
   "Parent element for Title and ASIN of similar products in other product groups "
   "Ancestry:  Cart"
   )
  (owning-merchant-id
   "The merchant ID that is associated with the promotion. OwningMerchantId is an updated ID for MerchantId. These IDs may or may not be the same."
   "Ancestry:  None"
   )
  (package-quantity
   "Quantity in a package"
   "Ancestry:  "
   "Variations/Item/ItemAttributes/"
   )
  (parameter
   "A parameter that an "
   " can or must use."
   "Ancestry:  Help"
   "operation"
   )
  (parent-asin
   "A parent ASIN for an item in Saved For Later or the Active areas of a cart.. "
   "Ancestry:  "
   "CartItem"
   "SavedForLaterItem"
   )
  (partner-name
   "For wedding registries, this is the name of the bride or groom."
   "Ancestry:  "
   "List/PartnerName"
   )
  (pearl-lustre
   "The luster of the pearl. "
   "Ancestry:  ItemAttributes"
   )
  (pearl-minimum-color
   "The pearl's minimum color. "
   "Ancestry:  ItemAttributes"
   )
  (pearl-shape
   "The shape of the pearl. "
   "Ancestry:  ItemAttributes"
   )
  (pearl-stringing-method
   "The means used to string the pearls. "
   "Ancestry:  ItemAttributes"
   )
  (pearl-surface-blemishes
   "A description of the surface blemishes on the pearl. "
   "Ancestry:  ItemAttributes"
   )
  (pearl-type
   "Pearl type is categorized by the shape and size of the pearl. "
   "Ancestry:  ItemAttributes"
   )
  (pearl-uniformity
   "Describes the uniformity of color of the pearl. "
   "Ancestry: ItemAttributes "
   )
  (phone-number
   "Phone number "
   "Ancestry:  ItemAttributes"
   )
  (photo-flash-type
   "Description of the camera's flash. "
   "Ancestry:  ItemAttributes"
   )
  (picture-format
   "Describes the format of an electronic image. "
   "Ancestry:  ItemAttributes"
   )
  (platform
   "Operating system. "
   "Ancestry:  ItemAttributes"
   )
  (postal-code
   "Postal code, such as a zip code. "
   "Ancestry:  ItemAttributes/Address"
   "Offers/Offer/OfferListing"
   )
  (price
   "A parent element for Amount, CurrencyCode, and FormattedPrice. In this case, the price is for an item in the Active or Saved For Later areas, respectively. "
   "Ancestry:  "
   "CartItem"
   "SavedForLaterItem"
   )
  (price-rating
   "Defines how expensive an item is. For example, for apparel, 1=under $15 2=$15-30 3=$30-45 4=over $45. "
   "Ancestry:  ItemAttributes"
   )
  (processor-count
   "The number of processors in an item. "
   "Ancestry:  ItemAttributes"
   )
  (product-group
   "The product category an item belongs to. The name of a category, such as sporting goods, to which an item in the cart belongs. The name of a category, such as sporting goods, to which an item in Saved For Later belongs."
   "Ancestry:  ItemAttributes/ProductGroup CartItem/ProductGroup SavedForLaterItem"
   )
  (product-type-subcategory
   "Subcategory of product type"
   "Ancestry:  Variations/Item/ItemAttributes"
   )
  (promotion
   "Container for one or more Details elements. "
   "Ancestry:  "
   "Offers/OfferPromotions"
   )
  (promotion-category
   "Specifies the kind of promotion. Valid values include FreeShipping, BuyAmountXGetAmountOffX, and ForEachAmountXGetAmountOffX. For more information, see "
   ". "
   "Ancestry:  "
   "Offers/OfferPromotions/Promotion/Summary"
   "Promotion Types"
   )
  (promotion-id
   "An alphanumeric token that uniquely identifies a promotion. "
   "Ancestry:  Offers/Offer/Promotions/Promotion/Summary"
   )
  (promotions
   "A container for one or more Promotion elements. "
   "Ancestry:  "
   "Offers/Offer"
   )
  (publication-date
   "Publication date. "
   "Ancestry:  ItemAttributes"
   )
  (publisher
   "Publisher "
   "Ancestry:  ItemAttributes"
   )
  (purchase-url
   "A URL that the customer must use to purchase the items in their cart. "
   "Ancestry:  Cart"
   )
  (quantity
   "The number of a particular item in a cart. Or, the number of items in stock.Or, the number of a particular item in Saved For Later."
   "Ancestry:  "
   "CartItem, SellerListing"
   "SavedForLaterItem"
   )
  (quantity-desired
   "The number of items the customer wants."
   "Ancestry:  ListFull, ListItems"
   )
  (quantity-received
   "The number of items the customer received."
   "Ancestry:  ListFull, ListItems"
   )
  (rating
   "A customer's rating in their review where a rating of 5 is the best. "
   "Ancestry: "
   "CustomerReviews/Review "
   )
  (reading-level
   "Describes the reading difficulty of the book. "
   "Ancestry:  ItemAttributes"
   )
  (registry-number
   "Number that identifies a registry, such as a wedding registry. "
   "Ancestry:  ListFull"
   )
  (region-code
   "A code that specifies a region. "
   "Ancestry:  ItemAttributes"
   )
  (registry-name
   "Name of a wedding or baby registry. "
   "Ancestry:  "
   "List/RegistryName"
   )
  (release-date
   "Date on which the item was was latest released. Items that have been released multiple times have both release dates and original release dates. See ReleaseDate."
   "Ancestry:  ItemAttributes"
   )
  (removable-memory
   "Specifies whether or not the memory is removable. "
   "Ancestry: ItemAttributes "
   )
  (request-id
   "A number that uniquely identifies a request "
   "Ancestry:  OperationRequest"
   )
  (resolution-modes
   "Specifies the resolution modes of the item. "
   "Ancestry:  ItemAttributes"
   )
  (response-group
   "An ECS response group."
   "Ancestry:  Help"
   )
  (ring-size
   "Size of a ring. "
   "Ancestry:  ItemAttributes"
   )
  (role
   "Role "
   "Ancestry:  ItemAttributes/Creator"
   )
  (running-time
   "The duration of a presentation. "
   "Ancestry:  ItemAttributes"
   )
  (sales-rank
   "Indicates how well an item is selling within its product category. The lower the number, the better the item has sold. "
   "Ancestry:  Item"
   )
  (saved-for-later-item
   "A positive integer that uniquely identifies an item in Saved For Later. "
   "Ancestry:  "
   "Cart/SavedForLaterItems "
   "Cart "
   )
  (scent-name
   "Scent of perfume or cologne"
   "Ancestry:  "
   "Variations/Item/ItemAttributes/"
   )
  (search-bin-set
   "A means by which to categorize results, such as price range."
   "Ancestry:  SearchBinSets"
   )
  (search-bin-sets
   "Parent element for SearchBins element. "
   "Ancestry:  SearchBinSets"
   )
  (secondary-cache-size
   "Size of the secondary cache. "
   "Ancestry:  ItemAttributes"
   )
  (seller-id
   "A positive integer that uniquely identifies the seller, or the seller of an item that is in either the Active or Saved For Later areas of a cart. "
   "Ancestry:  "
   "SavedForLaterItem"
   "SellerListing/Seller"
   "CartItem Offers/Offer/Seller"
   )
  (seller-legal-name
   "Seller's legal name. This can differ from the Name value. For example, Amazon could be the value for the Name, but Amazon.com for the Legalname. "
   "Ancestry: Seller "
   )
  (seller-nickname
   "The nickname of the seller of an item in Saved For Later. The nickname is often a less formal version of the merchant's official name, such as, MyCompany, as opposed to MyCompany Ltd.."
   "Ancestry:  CartItem, Seller"
   )
  (setting-type
   "The setting type used in a piece of jewelry. "
   "Ancestry: ItemAttributes "
   )
  (shaft-material-type
   "Material the shaft is composed of"
   "Ancestry:  "
   "Variations/Item/ItemAttributes/"
   )
  (shipment-items
   "The items in a specific shipment. "
   "Ancestry:  "
   "Transaction/Shipments/Shipment "
   )
  (shipments
   "The shipments "
   "Ancestry:  Transaction"
   )
  (similar-products
   "Parent element for Title and ASIN of similar products in the same product group "
   "Ancestry:  Cart"
   )
  (similar-viewed-products
   "Parent element for Title and ASIN of similar products in the same product group that have been viewed"
   "Ancestry:  Cart"
   )
  (single-merchant-id
   "ID of a merchant that applies across all of the variations of an item. "
   "Ancestry:  VariationSummary"
   )
  (size
   "Size "
   "Ancestry:  ItemAttributes"
   )
  (size-per-pearl
   "Size of the pearl(s) in a piece of jewelry. "
   "Ancestry:  ItemAttributes"
   )
  (sku
   "Stock Keeping Unit (SKU) is a number that uniquely identifies an item."
   "Ancestry:  ItemAttributes"
   "SellerListing"
   )
  (small-image
   "Container for a small image. Thumbnail and Small images are the same size."
   "Ancestry:  "
   "ImageSets/ImageSet"
   )
  (sound-card-description
   "Describes a computer's sound card. "
   "Ancestry:  ItemAttributes"
   )
  (source
   "Where the review was entered. Typically, this is Amazon.com. It is possible, however, for customers to enter reviews from other sites. "
   "Ancestry:  "
   "EditorialReviews/EditorialReview"
   )
  (speaker-description
   "Describes the item's speakers. "
   "Ancestry:  ItemAttributes"
   )
  (special-features
   "Describes an item's special features. "
   "Ancestry:  ItemAttributes"
   )
  (start-date
   "The data an item goes on sale. Or, specifies the first day and beginning time (GMT) of the promotion. "
   "Ancestry:  "
   "SellerListing, Offers/Offer/Promotions/Promotion/Summary "
   )
  (state
   "State "
   "Ancestry:  ItemAttributes"
   )
  (status
   "Specifies whether an item is for sale (Open) or not (Closed). To determine if an item is truly available, this value must be Open, Quantity must be greater than zero and also the value of QuantityAllocated."
   "Ancestry:  "
   )
  (stone-clarity
   "The clarity of a gem. "
   "Ancestry:  ItemAttributes"
   )
  (stone-color
   "The color of a gem. "
   "Ancestry:  ItemAttributes"
   )
  (stone-cut
   "The cut of a gem. "
   "Ancestry:  ItemAttributes"
   )
  (stone-shape
   "The shape of a gem. "
   "Ancestry:  ItemAttributes"
   )
  (stone-weight
   "The number of karats of a gem. "
   "Ancestry:  ItemAttributes"
   )
  (store-id
   "A number that uniquely identifies the seller's store. "
   "Ancestry: "
   "SellerListing/Seller "
   )
  (store-name
   "The name of the seller's store. "
   "Ancestry:  "
   "SellerListing/Seller "
   )
  (studio
   "Studio "
   "Ancestry:  ItemAttributes"
   )
  (style
      "Style"
    "Ancestry:  "
    "Variations/Item/ItemAttributes/"
    )
  (subject
   "A book's Subject description, which characterizes the book's content."
   "Ancestry:  Subjects"
   )
  (sub-condition
   "The condition of an item. This tag helps to further describes items that are not new. Values, for example, are mint, very good, and good. "
   "Ancestry:  SellerListing/SubCondition"
   )
  (subscription-length
   "The duration of a subscription. "
   "Ancestry:  ItemAttributes"
   )
  (summary
   "A short summary of the content of the review. Or, a container for all promotion information, including the elements BenefitDescription, Category, EndDate, EligibilityRequirementsDescription, PromotionId, StartDate, and TermsAndConditions. "
   "Ancestry:  CustomerReviews/Review/Summary, Offers/Offer/Promotions/Promotion"
   )
  (supported-image-type
   "The image formats supported by the item. "
   "Ancestry:  ItemAttributes"
   )
  (swatch-image
   "Container for a swatch image, which is smaller than a Small image."
   "Ancestry:  "
   "ImageSets/ImageSet"
   )
  (system-bus-speed
   "The bus speed of the motherboard. "
   "Ancestry:  ItemAttributes"
   )
  (system-memory-size
   "The size of the item's RAM. "
   "Ancestry:  ItemAttributes"
   )
  (system-memory-size-max
   "The maximum RAM size an item can be expanded to. "
   "Ancestry:  ItemAttributes"
   )
  (system-memory-type
   "The type of RAM memory used. "
   "Ancestry:  ItemAttributes"
   )
  (tag
   "A customer-created tag for an item, such as BobsBestMovie. "
   "Ancestry:  TaggedGuides, TaggedItems, TaggedListmaniaLists, Tag"
   )
  (tagged-items
   "Container for Tag elements, such as Item and DistinctUsers."
   "Ancestry:  TaggedGuides, TaggedItems, TaggedListmaniaLists, Tag"
   )
  (tag-name
   "Tag name."
   "Ancestry:  TaggedGuides, TaggedItems, TaggedListmaniaLists, Tag"
   )
  (tags
   "Container for all Tag elements."
   "Ancestry:  None"
   )
  (tag-type
   "Describes the entity tagged, for example, Items."
   "Ancestry:  TaggedGuides, TaggedItems, TaggedListmaniaLists, Tag"
   )
  (terms-and-conditions
   "Specifies the terms and conditions of the promotion. "
   "Ancestry:  Offers/Offer/Promotions/Promotion/Summary"
   )
  (theatrical-release-date
   "The date of a theatrical release. "
   "Ancestry:  ItemAttributes"
   )
  (thumbnail-image
   "Container for a Thumbnail image. Thumbnail and Small images are the same size. "
   "Ancestry:  ImageSets/ImageSet"
   )
  (time
   "The time an entity was tagged. "
   "Ancestry:  TaggedItems, Tags"
   )
  (tiny-image
   "Container for a tiny image "
   "Ancestry:  ImageSets/ImageSet"
   )
  (title
   "Title or the name of the accessory, item, new release, similar products in other product groups, similar products in the same product group, similar products in the same product group that have been viewed "
   "Ancestry:  ItemAttributes"
   "Accessories/Accessory SavedForLaterItem Cart/NewReleases/NewRelease/Title CartItem OtherCategoriesSimilarProduct SimilarProduct SimilarViewedProduct TopSellers/TopSeller "
   )
  (total-collectible
   "The total number of collectible items for sale. "
   "Ancestry:  OfferSummary"
   )
  (total-diamond-weight
   "Total weight of all of the diamonds in a piece of jewelry. "
   "Ancestry:  ItemAttributes"
   )
  (total-external-bays-free
   "The number of free external bays in a computer. "
   "Ancestry:  ItemAttributes"
   )
  (total-feedback
   "Average rating of the customers' feedback about the seller. "
   "Ancestry:  "
   "Offers/Offer/Seller"
   )
  (total-feedback-pages
   "Specifies the number of pages of customer feedback. Up to twenty customer reviews are returned per page. "
   "Ancestry: Seller "
   )
  (total-firewire-ports
   "The number of firewire ports an item has. "
   "Ancestry:  ItemAttributes"
   )
  (total-gem-weight
   "Total weight of all of the gems in a piece of jewelry. "
   "Ancestry:  ItemAttributes"
   )
  (total-internal-bays-free
   "Number of internal bays free in a computer "
   "Ancestry: ItemAttributes "
   )
  (total-items
   "The total number of items found on the list. Each page holds up to ten items. "
   "Ancestry: "
   "Items/TotalItems "
   )
  (total-metal-weight
   "Total weight of all of the metal in a piece of jewelry. "
   "Ancestry:  ItemAttributes"
   )
  (total-new
   "The total number of new items for sale. "
   "Ancestry: OfferSummary "
   )
  (total-ntscpal-ports
   "Total number of NTSC and PAL video ports. "
   "Ancestry:  ItemAttributes"
   )
  (total-offer-pages
   "Number of pages of offers. Up to twenty offers are returned per page. By default, the first ten offers are returned in the response. "
   "Ancestry:  Offers"
   )
  (total-offers
   "Total number of offers. "
   "Ancestry:  Offers"
   )
  (total-pages
   "The total number of pages found in a response. Each page holds up to ten items. "
   "Ancestry: Items "
   )
  (total-parallel-ports
   "Total number of parallel ports. "
   "Ancestry:  ItemAttributes"
   )
  (total-pc-card-slots
   "Total number of PC card slots in a computer. "
   "Ancestry: ItemAttributes "
   )
  (total-pci-slots-free
   "Number of free PCI card slots in a computer. "
   "Ancestry: ItemAttributes "
   )
  (total-ratings
   "Catalogs all customer ratings of the seller. "
   "Ancestry: "
   "Seller/TotalRatings "
   )
  (total-refurbished
   "The total number of refurbished items for sale. "
   "Ancestry:  OfferSummary"
   )
  (total-results
   "Total number of items found by the request. Only ten are returned at a time."
   "Ancestry:  "
   "Items, List"
   )
  (total-review-pages
   "The number of pages of reviews found. Up to ten reviews are returned in a response "
   "Ancestry:  CustomerReviews"
   )
  (total-reviews
   "Total number of reviews of an item. "
   "Ancestry:  CustomerReviews"
   )
  (totals
   "Container object for all other Total* elements, including Total, SubTotal, Tax, ShippingCharges, Promotion, and so forth. "
   "Ancestry:  Transaction"
   )
  (total-serial-ports
   "Total number of serial ports on an item. "
   "Ancestry:  ItemAttributes"
   )
  (total-s-video-out-ports
   "Total number of SVideo out connectors. "
   "Ancestry:  ItemAttributes"
   )
  (total-times-read
   "The number of times a list has been viewed. "
   "Ancestry:  List"
   )
  (total-usages
   "The number of times an entity has been accessed. "
   "Ancestry:  TaggedGuides, TaggedItems, TaggedListmaniaLIsts, Tags"
   )
  (total-usb-ports
   "Total number of USB2 ports on an item. "
   "Ancestry:  ItemAttributes"
   )
  (total-usb2-ports
   "Total number of USB1 ports on an item. "
   "Ancestry:  ItemAttributes"
   )
  (total-used
   "The total number of used items for sale. "
   "Ancestry: OfferSummary "
   )
  (total-vga-out-ports
   "Total number of VGA (video) outputs on an item. "
   "Ancestry:  ItemAttributes"
   )
  (total-votes
   "The total number of review votes cast. "
   "Ancestry: CustomerReviews/Review/TotalVotes "
   )
  (track
   "Refers to each track on a CD. On a music CD, each track corresponds to a song. "
   "Ancestry: "
   "Tracks/Disc "
   )
  (transaction-date
   "The date on which the transaction began of the form yyyy-mm-ddThh.mm.ss, for example, 2005-06-08T19:19:27."
   "Ancestry:  Transaction"
   )
  (transaction-date-epoch
   "The date on which the transaction began in the form of epoch seconds."
   "Ancestry:  Transaction"
   )
  (transaction-id
   "A series of dash-separated integers, such as 111-222-333, that uniquely identify a transaction. "
   "Ancestry:  Transaction"
   )
  (transaction-item
   "Container for all of the information related to a specified transaction. "
   "Ancestry:  Transaction"
   )
  (transaction-item-id
   "A string that uniquely identifies a transaction item. "
   "Ancestry:  Transaction/TransactionItems/TransactionItem "
   )
  (transaction-items
   "Container for TransactionItem "
   "Ancestry:  Transaction"
   )
  (type
   "Type "
   "Ancestry:  ItemAttributes/Languages/Language"
   )
  (unit
   "This is the unit of time returned by the element, "
   "."
   "Ancestry:  ItemAttributes"
   "MaximumShutterSpeed"
   )
  (upc
   "Universal Product Code, which is a 12 digit number, 6 of which represents an item's manufacturer. These numbers are translated into a bar code that is printed on an item or its packaging."
   "Ancestry:  ItemAttributes"
   "SellerListing"
   )
  (url
   "URL of an image. "
   "Ancestry:  "
   "Variations/Item/ImageSets/ImageSet/LargeImage"
   "Variations/Item/ImageSets/ImageSet/SwatchImage"
   "Variations/Item/ImageSets/ImageSet/MediumImage"
   "Variations/Item/ImageSets/ImageSet/SmallImage"
   )
  (url-encoded-hmac
   "A URL-encoded version of the HMAC that can be used directly in a request. "
   "Ancestry: Cart "
   )
  (user-agent
   "The client application, for example, web browsers, search engine crawlers, mobile phones, screen readers and braille browsers. The text associated with UserAgent contains the application name, version, host operating system, and language."
   "Ancestry:  OperationRequest"
   )
  (user-id
   "An alphanumeric token that uniquely identifies the customer that tagged the entity."
   "Ancestry:  "
   )
  (value
   "The value of a parameter passed in the request. Or, the value for item in bin. "
   "Ancestry:  TaggedItems, TaggedListmaniaLists, Tags"
   )
  (variation-denomination
   "Describes the variation. Typically used in the Electronics product category. "
   "Ancestry:  ItemAttributes"
   )
  (variation-description
   "Describes the variation. A variation is a particular instance of an item, for example, a specific size and color of a shirt. "
   "Ancestry:  ItemAttributes"
   )
  (variation-dimension
   "Container for dimensions"
   "Ancestry:  "
   "Variations/VariationDimensions/"
   )
  (version
   "ECS version number."
   "Ancestry:  Help"
   )
  (warranty
   "Warranty terms "
   "Ancestry:  ItemAttributes"
   )
  (watch-movement-type
   "The type of movement used in a watch. "
   "Ancestry:  ItemAttributes"
   )
  (water-resistance-depth
   "How deep a watch can go in water before water leaks into it. "
   "Ancestry:  ItemAttributes"
   )
  (weight
   "Weight "
   "Ancestry:  ItemAttributes/ItemDimensions ItemAttributes/PackageDimensions/Weight ItemAttributes"
   )
  (width
   "ItemAttributes/ItemDimensions ItemAttributes/PackageDimensions/Width ItemAttributes "
   "Width "
   "Ancestry:  "
   "Variations/Item/ImageSets/ImageSet/SwatchImage"
   "Variations/Item/ImageSets/ImageSet/LargeImage"
   "Variations/Item/ImageSets/ImageSet/SmallImage"
   "Variations/Item/ImageSets/ImageSet/MediumImage"
   )
  (will-ship-expedited
   "Specifies whether seller can ship items faster than normal delivery."
   "Ancestry:  "
   "SellerListing/WillShipExpedited"
   )
  (will-ship-international
   "Specifies whether seller will ship item to international destinations."
   "Ancestry:  "
   "SellerListing/WillShipInternational"
   )
  (wish-list-id
   "An alphanumeric token that uniquely identifies a wish list."
   "Ancestry:  "
   "CustomerFull, CustomerLists"
   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A2S API Reference > Search Index Values
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-syntax define-aws-search-indices
  (syntax-rules ()
    ((_ (locale ...) (index a ...) ...)
     (begin
       '(locale ...)
       '(index a ...)
       ...))))

(define-aws-search-indices
  ;; locales --------------------------------------------------------------
  (us uk de jp fr ca)
  ;; indices ------------------------------------------------------------------------
  (all #t #f #f #f #f #f)
  (apparel #t #t #t #t #f #f)
  (automotive #t #f #f #f #f #f)
  (baby #t #t #t #t #f #f)
  (beauty #t #f #f #f #f #f)
  (blended #t #t #t #t #t #t)
  (books #t #t #t #t #t #t)
  (classical #t #t #t #t #t #t)
  (digital-music #t #f #f #f #f #f)
  (dvd #t #t #t #t #t #t)
  (electronics #t #t #t #t #t #f)
  (foreign-books #f #f #t #t #t #t)
  (gourmet-food #t #f #f #f #f #f)
  (grocery #t #f #f #f #f #f)
  (health-personal-care #t #t #t #t #f #f)
  (hobbies #f #f #f #t #f #f)
  (home-garden #t #t #t #f #f #f)
  (industrial #t #f #f #f #f #f)
  (jewelry #t #f #f #f #f #f)
  (kindle-store #f #f #f #f #f #f)
  (kitchen #t #t #t #t #t #f)
  (magazines #t #f #t #f #f #f)
  (merchants #t #f #f #f #f #f)
  (miscellaneous #t #f #f #f #f #f)
  (mp3-downloads #t #f #f #f #f #f)
  (music #t #t #t #t #t #t)
  (musical-instruments #t #f #f #f #f #f)
  (music-tracks #t #t #t #t #t #f)
  (office-products #t #f #f #f #f #f)
  (outdoor-living #t #t #t #f #f #f)
  (pc-hardware #t #f #t #f #f #f)
  (pet-supplies #t #f #f #f #f #f)
  (photo #t #f #t #f #f #f)
  (silver-merchant #t #f #f #f #f #f)
  (software #t #t #t #t #t #t)
  (software-video-games #f #t #t #f #t #t)
  (sporting-goods #t #t #t #t #f #f)
  (tools #t #t #t #f #f #f)
  (toys #t #t #t #t #t #f)
  (vhs #t #t #t #t #t #t)
  (video #t #t #t #t #t #t)
  (video-games #t #t #t #t #t #t)
  (watches #f #t #t #f #t #f)
  (wireless #t #f #f #f #f #f)
  (wireless-accessories #t #f #f #f #f #f)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A2S API Reference > ItemSearch Sort Values By Locale
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; - For an ItemSearch requests that do not use the BrowseNode parameter, results are sorted by Relevance.
;; - For an ItemSearch requests that do use the BrowseNode parameter, results are sorted by BestSeller ranking.

(define-syntax define-aws-item-search-sort-values
  (syntax-rules ()
    ((_ (locale (index (value description) ...) ...) ...)
     (begin
       '(locale (index (value description) ...)
                ...)
       ...))))

(define-aws-item-search-sort-values
  (ca ; ----------------------------------------------------------------------
   (books
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (daterank "Pub Date: New to Old")
    (titlerank "Alphabetical: A to Z")
    )
   (classical
    (salesrank "Bestselling")
    (titlerank "Alphabetical: A to Z")
    (orig-rel-date "Rel Date: New to Old")
    )
   (dvd
    (salesrank "Bestselling")
    (titlerank "Alphabetical: A to Z")
    )
   (foreignbooks
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: newer to older")
    (titlerank "Alphabetical: A to Z")
    )
   (music
    (salesrank "Bestselling")
    (titlerank "Alphabetical: A to Z")
    (orig-rel-date "Rel Date: New to Old")
    )
   (software
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-daterank| "Rel Date: Old to New")
    )
   (softwarevg
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-daterank| "Rel Date: Old to New")
    )
   (vhs
    (salesrank "Bestselling")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (video
    (salesrank "Bestselling")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (videogames
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   )
  (de ; ----------------------------------------------------------------------
   (apparel
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling to worst selling")
    )
   (baby
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling to worst selling")
    )
   (books
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (classical
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (pubdate "Publication date: most recent to oldest")
    (|-pubdate| "Publication date: oldest to most recent")
    (publication_date "Publication date: most recent to oldest")
    (|-publication_date| "Publication date: oldest to most recent")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (dvd
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (electronics
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (foreignbooks
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (healthpersonalcare
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (homegarden
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (kitchen
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling to worst selling")
    )
   (magazines
    (salesrank "Bestselling")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (music
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (pubdate "Publication date: most recent to oldest")
    (|-pubdate| "Publication date: oldest to most recent")
    (publicationdate "Publication date: most recent to oldest")
    (|-publicationdate| "Publication date: oldest to most recent")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (musictracks
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (outdoorliving
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (pchardware
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (photo
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (software
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (|-date| "Publication date: newer to older")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (sportinggoods
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (release-date "Release date: older to newer")
    (|-release-date| "Release date: newer to older")
    )
   (softwarevideogames
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (|-date| "Publication date: newer to older")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (tools
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-pricerank| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (toys
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (|-date| "Pub Date: Old to New")
    (titlerank "Alphabetical: A to Z")
    )
   (vhs
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (video
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (videogames
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (|-date| "Publication date: newer to older")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (watches
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   )
  (fr ; ----------------------------------------------------------------------
   (books
    (salesrank "Bestselling to worse selling")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (|-daterank| "Pub Date: Old to New")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (classical
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (price "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: A to Z")
    )
   (dvd
    (amzrank "Alphabetical: A to Z")
    (availability "Most to least available")
    (salesrank "Bestselling")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (electronics
    (salesrank "Bestselling")
    (price "Price: Low to high")
    (|-price| "Price: High to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (foreignbooks
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (|-daterank| "Publication date: Old to New")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (kitchen
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling to worst selling")
    )
   (music
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (musictracks
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (software
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-pricerank| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: A to Z")
    )
   (softwarevg
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-pricerank| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: A to Z")
    (|-date| "Rel Date: Old to New")
    )
   (vhs
    (amzrank "Most to least available")
    (availability "Most to least available")
    (salesrank "Bestselling")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (video
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (videogames
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (date "Rel Date: new to old")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (watches
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling to worst selling")
    )
   )
  (jp ; ----------------------------------------------------------------------
   (apparel
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling to worst selling")
    )
   (baby
    (psrank "Bestseller ranking taking into consideration projected sales.The lower the value, the better the sales.")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    )
   (beauty
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (|-launch-date| "Newest arrivals")
    (sale-flag "On sale")
    )
   (books
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: newer to older")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (classical
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (|-pricerank| "Price: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (|-orig-rel-date| "Release date: newer to older")
    (orig-rel-date "Release date: older to newer")
    (releasedate "Release date: newer to older")
    (|-releasedate| "Release date: older to newer")
    )
   (dvd
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (|-pricerank| "Price: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (|-orig-rel-date| "Release date: newer to older")
    (orig-rel-date "Release date: older to newer")
    (releasedate "Release date: older to newer")
    (|-releasedate| "Release date: newer to older")
    )
   (electronics
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (|-releasedate| "Release date: newer to older")
    (releasedate "Release date: older to newer")
    )
   (foreignbooks
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: newer to older")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (healthpersonalcare
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (hobbies
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (release-date "Release date: older to newer")
    (|-release-date| "Release date: newer to older")
    (mfg-age-min "Minimum age of user sorted from youngest to oldest.")
    (|-mfg-age-min| "Minimum age of user sorted from oldest to youngest.")
    )
   (kitchen
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (|-release-date| "Release date: newer to older")
    (release-date "Release date: older to newer")
    )
   (music
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (|-pricerank| "Price: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (|-orig-rel-date| "Release date: newer to older")
    (orig-rel-date "Release date: older to newer")
    (releasedate "Release date: older to newer")
    (|-releasedate| "Release date: newer to older")
    )
   (musictracks
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (software
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (|-release-date| "Release date: newer to older")
    (release-date "Release date: older to newer")
    )
   (sportinggoods
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (releasedate "Release date: older to newer")
    (|-releasedate| "Release date: newer to older")
    )
   (toys
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (|-release-date| "Release date: newer to older")
    (release-date "Release date: older to newer")
    )
   (vhs
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (|-pricerank| "Price: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (|-orig-rel-date| "Release date: newer to older")
    (orig-rel-date "Release date: older to newer")
    (releasedate "Release date: older to newer")
    (|-releasedate| "Release date: newer to older")
    )
   (video
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (pricerank "Price: low to high")
    (|-pricerank| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (|-orig-rel-date| "Release date: newer to older")
    (orig-rel-date "Release date: older to newer")
    (releasedate "Release date: older to newer")
    (|-releasedate| "Release date: newer to older")
    )
   (videogames
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (|-release-date| "Release date: newer to older")
    (release-date "Release date: older to newer")
    )
   (watches
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   )
  (uk ; ----------------------------------------------------------------------
   (apparel
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling to worst selling")
    )
   (baby
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling to worst selling")
    )
   (books
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: older to newer")
    (pubdate "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (classical
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (inverse-pricerank "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (dvd
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: older to newer")
    (releasedate "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (electronics
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (healthpersonalcare
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (daterank "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (homegarden
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (daterank "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (kitchen
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (daterank "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (music
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (inverse-pricerank "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (musictracks
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (outdoorliving
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (daterank "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (software
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (softwarevideogames
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (toys
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (|-mfg-age-min| "Age: high to low")
    (mfg-age-min "Age: low to high")
    )
   (vhs
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: older to newer")
    (releasedate "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (video
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: older to newer")
    (releasedate "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (videogames
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: older to newer")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (watches
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   )
  (us ; ----------------------------------------------------------------------
   (all
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverseprice "Price: high to low")
    (|-launch-date| "Newest arrivals")
    (sale-flag "On sale")
    )
   (apparel
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (automotive
    (psrank "Bestseller ranking taking into consideration projected sales.The lower the value, the better the sales.")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    )
   (baby
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (|-launch-date| "Newest arrivals")
    (sale-flag "On sale")
    )
   (beauty
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (daterank "Publication date: newer to older")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (books
    (psrank "Bestseller ranking taking into consideration projected sales.The lower the value, the better the sales.")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (orig-rel-date "Release date: newer to older")
    (|-orig-rel-date| "Release date: older to newer")
    (releasedate "Release date: newer to older")
    (|-releasedate| "Release date: older to newer")
    )
   (classical
    (songtitlerank "Most popular")
    (uploaddaterank "Date added")
    )
   (digitalmusic
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-video-release-date| "Release date: newer to older")
    (releasedate "Release date: newer to older")
    )
   (dvd
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    )
   (electronics
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverseprice "Price: high to low")
    (launch-date "Newest arrivals")
    (sale-flag "On sale")
    )
   (gourmetfood
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverseprice "Price: high to low")
    (launch-date "Newest launches first")
    (sale-flag "On sale")
    )
   (grocery
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverseprice "Price: high to low")
    (launch-date "Newest arrivals")
    (sale-flag "On sale")
    )
   (healthpersonalcare
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (homegarden
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (industrial
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverseprice "Price: high to low")
    (launch-date "Newest arrivals")
    )
   (jewelry
    (daterank "Publication date: newer to older")
    (|-edition-sales-velocity| "Quickest to slowest selling products.")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (reviewrank "Average customer review: high to low")
    )
   (kindlestore
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (kitchen
    (subslot-salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (daterank "Publication date: newer to older")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (magazines
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverseprice "Price: high to low")
    (|-launch-date| "Newest arrivals")
    (sale-flag "On sale")
    )
   (merchants
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (miscellaneous
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (releasedate "Release date: most recent to oldest")
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    )
   (mp3downloads
    (psrank "Bestseller ranking taking into consideration projected sales.The lower the value, the better the sales.")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    (artistrank "Artist name: A to Z")
    (orig-rel-date "Original release date of the item listed from newer to older. See release-date, which sorts by the latest release date.")
    (release-date "Sorts by the latest release date from newer to older. See orig-rel-date, which sorts by the original release date.")
    (releasedate "Release date: most recent to oldest")
    (|-releasedate| "Release date: oldest to most recent")
    )
   (music
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (|-launch-date| "Newest arrivals")
    (sale-flag "On sale")
    )
   (musicalinstruments
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (musictracks
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (reviewrank "Average customer review: high to low")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    )
   (officeproducts
    (psrank "Bestseller ranking taking into consideration projected sales.The lower the value, the better the sales.")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (outdoorliving
    (psrank "Bestseller ranking taking into consideration projected sales.The lower the value, the better the sales.")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    )
   (pchardware
    (|+pmrank| "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (petsupplies
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (photo
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverseprice "Price: high to low")
    (|-launch-date| "Newest arrivals")
    (sale-flag "On sale")
    )
   (silvermerchants
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    )
   (software
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (pricerank "Price: low to high")
    (inverseprice "Price: high to low")
    (launch-date "Newest arrivals")
    (sale-flag "On sale")
    )
   (sportinggoods
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (tools
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-age-min| "Age: high to low")
    )
   (toys
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-video-release-date| "Release date: newer to older")
    )
   (unboxvideo
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-video-release-date| "Release date: newer to older")
    (|-releasedate| "Release date: newer to older")
    )
   (vhs
    (relevancerank "Items ranked according to the following criteria: how often the keyword appears in the description, where the keyword appears, for example, the ranking is higher when keywords are found in titles, and, if there are multiple keywords, how closely they occur in descriptions, and, finally, how often customers purchased the products they found using the keyword.")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    (|-video-release-date| "Release date: newer to older")
    (|-releasedate| "Release date: newer to older")
    )
   (video
    (pmrank "Featured items")
    (salesrank "Bestselling")
    (price "Price: low to high")
    (|-price| "Price: high to low")
    (titlerank "Alphabetical: A to Z")
    )
   (videogames
    (daterank "Publication date: newer to older")
    (pricerank "Price: low to high")
    (inverse-pricerank "Price: high to low")
    (reviewrank "Average customer review: high to low")
    (salesrank "Bestselling")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   (wireless
    (psrank "Bestseller ranking taking into consideration projected sales.The lower the value, the better the sales.")
    (salesrank "Bestselling")
    (titlerank "Alphabetical: A to Z")
    (|-titlerank| "Alphabetical: Z to A")
    )
   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A2S API Reference > Search Index and ItemSearch Parameter Combinations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-syntax define-aws-search-index-params
  (syntax-rules ()
    ((_ (locale (index param ...) ...) ...)
     (begin
       '(locale (index param ...) ...)
       ...))))

(define-aws-search-index-params
  (ca ; ----------------------------------------------------------------------
   (blended
    author
    browse-node
    condition
    count
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    power
    publisher
    sort
    title
    )
   (books
    author
    browse-node
    composer
    condition
    conductor
    count
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    music-label
    sort
    title
    )
   (classical
    actor
    audience-rating
    browse-node
    condition
    count
    director
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    publisher
    sort
    title
    )
   (dvd
    browse-node
    condition
    count
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    power
    publisher
    sort
    title
    )
   (foreignbooks
    artist
    browse-node
    condition
    count
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    music-label
    power
    sort
    title
    )
   (music
    browse-node
    condition
    count
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (software
    author
    browse-node
    condition
    count
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (softwarevideogames
    actor
    browse-node
    condition
    count
    director
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    publisher
    sort
    title
    )
   (vhs
    actor
    audience-rating
    browse-node
    condition
    count
    director
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    publisher
    sort
    title
    )
   (video
    author
    brand
    browse-node
    condition
    count
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   )
  (de ; ----------------------------------------------------------------------
   (apparel
    author
    availability
    brand
    browse-node
    condition
    count
    delivery-method
    format
    ispu-postal-code
    item-page
    keywords
    magazines
    manufacturer
    merchant-id
    performer
    postal-code
    sort
    state
    title
    )
   (baby
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    title
    )
   (blended
    author
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    power
    publisher
    sort
    title
    )
   (books
    artist
    availability
    browse-node
    composer
    conductor
    count
    format
    item-page
    keywords
    magazines
    music-label
    orchestra
    performer
    postal-code
    sort
    state
    title
    )
   (classical
    actor
    availability
    browse-node
    composer
    conductor
    count
    format
    item-page
    keywords
    magazines
    music-label
    orchestra
    performer
    postal-code
    sort
    state
    title
    )
   (dvd
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (electronics
    author
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    publisher
    sort
    title
    )
   (foreignbooks
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (healthpersonalcare
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (homegarden
    author
    availability
    brand
    browse-node
    condition
    count
    delivery-method
    format
    ispu-postal-code
    item-page
    keywords
    magazines
    manufacturer
    merchant-id
    performer
    postal-code
    sort
    state
    title
    )
   (kitchen
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    publisher
    sort
    title
    )
   (magazines
    artist
    availability
    browse-node
    count
    format
    item-page
    keywords
    magazines
    music-label
    performer
    postal-code
    sort
    state
    title
    )
   (music
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    )
   (musictracks
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (outdoorliving
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (pchardware
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (photo
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (software
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (softwarevideogames
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (sportingoods
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (tools
    browse-node
    condition
    count
    item-page
    keywords
    sort
    merchant-id
    title
    )
   (toys
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (vhs
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (video
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    mpaa-rating
    sort
    title
    )
   (videogames
    browse-node
    condition
    item-page
    keywords
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   )
  (fr ; ----------------------------------------------------------------------
   (blended
    author
    browse-node
    condition
    count
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    power
    publisher
    sort
    title
    )
   (books
    actor
    availability
    browse-node
    composer
    conductor
    count
    format
    item-page
    keywords
    magazines
    music-label
    orchestra
    performer
    postal-code
    sort
    state
    title
    )
   (classical
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (dvd
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (electronics
    author
    browse-node
    condition
    count
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    power
    publisher
    sort
    title
    )
   (foreignbooks
    author
    availability
    brand
    browse-node
    condition
    count
    delivery-method
    format
    ispu-postal-code
    item-page
    keywords
    magazines
    manufacturer
    merchant-id
    performer
    postal-code
    sort
    state
    title
    )
   (kitchen
    artist
    availability
    browse-node
    count
    format
    item-page
    keywords
    magazines
    music-label
    performer
    postal-code
    sort
    state
    title
    )
   (music
    browse-node
    condition
    count
    item-page
    keywords
    merchant-id
    minimum-price
    sort
    )
   (musictracks
    browse-node
    condition
    count
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (software
    author
    browse-node
    condition
    count
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (softwarevideogames
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (vhs
    actor
    audience-rating
    availability
    browse-node
    director
    item-page
    keywords
    postal-code
    publisher
    sort
    state
    title
    )
   (video
    author
    brand
    browse-node
    condition
    count
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (videogames
    browse-node
    condition
    item-page
    keywords
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   )
  (jp ; ----------------------------------------------------------------------
   (apparel
    author
    availability
    brand
    browse-node
    condition
    count
    delivery-method
    format
    ispu-postal-code
    item-page
    keywords
    magazines
    manufacturer
    merchant-id
    performer
    postal-code
    sort
    state
    title
    )
   (baby
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    title
    )
   (blended
    author
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    power
    publisher
    sort
    title
    )
   (books
    actor
    availability
    browse-node
    composer
    conductor
    count
    format
    item-page
    keywords
    magazines
    music-label
    orchestra
    performer
    postal-code
    sort
    state
    title
    )
   (classical
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (dvd
    brand
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (electronics
    author
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    publisher
    sort
    title
    )
   (foreignbooks
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (healthpersonalcare
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (hobbies
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (kitchen
    artist
    availability
    browse-node
    count
    format
    item-page
    keywords
    magazines
    music-label
    performer
    postal-code
    sort
    state
    title
    )
   (music
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    )
   (musictracks
    brand
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (software
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (sportingoods
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (toys
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (vhs
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (video
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (videogames
    browse-node
    condition
    item-page
    keywords
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   )
  (uk ; ----------------------------------------------------------------------
   (blended
    author
    availability
    brand
    browse-node
    condition
    count
    delivery-method
    format
    ispu-postal-code
    item-page
    keywords
    magazines
    manufacturer
    merchant-id
    performer
    postal-code
    sort
    state
    title
    )
   (apparel
    author
    availability
    brand
    browse-node
    condition
    count
    delivery-method
    format
    ispu-postal-code
    item-page
    keywords
    magazines
    manufacturer
    merchant-id
    performer
    postal-code
    sort
    state
    title
    )
   (baby
    author
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    power
    publisher
    sort
    title
    )
   (books
    actor
    availability
    browse-node
    composer
    conductor
    count
    format
    item-page
    keywords
    magazines
    music-label
    orchestra
    performer
    postal-code
    sort
    state
    title
    )
   (classical
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (dvd
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (electronics
    brand
    browse-node
    condition
    item-page
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (healthpersonalcare
    )
   (homegarden
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (kitchen
    artist
    availability
    browse-node
    count
    format
    item-page
    keywords
    magazines
    music-label
    performer
    postal-code
    sort
    state
    title
    )
   (music
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    )
   (musictracks
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (outdoorliving
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (software
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (softwarevideogames
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (toys
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (vhs
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (video
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (videogames
    browse-node
    condition
    item-page
    keywords
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   )
  (us ; ----------------------------------------------------------------------
   (all
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   (apparel
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   (automotive
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    title
    )
   (baby
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   (beauty
    author
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    power
    publisher
    sort
    state
    text-stream
    title
    )
   (blended
    artist
    author
    availability
    browse-node
    composer
    conductor
    count
    format
    item-page
    keywords
    magazines
    music-label
    orchestra
    performer
    postal-code
    sort
    state
    title
    )
   (books
    actor
    browse-node
    condition
    director
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    publisher
    sort
    state
    title
    )
   (classical
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (digitalmusic
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   (mp3downloads
    browse-node
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   (dvd
    brand
    browse-node
    condition
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    item-page
    keywords
    sort
    state
    title
    )
   (electronics
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   (gourmetfood
    brand
    browse-node
    condition
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    item-page
    keywords
    sort
    state
    title
    )
   (healthpersonalcare
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    publisher
    state
    sort
    title
    )
   (home-garden
    keywords
    title
    browse-node
    artist
    author
    actor
    director
    audience-rating
    manufacturer
    music-label
    composer
    publisher
    brand
    conductor
    city
    neighborhood
    orchestra
    text-stream
    )
   (industrial
    brand
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    title
    )
   (jewelry
    artist
    availability
    browse-node
    count
    format
    item-page
    keywords
    magazines
    music-label
    performer
    postal-code
    sort
    state
    title
    )
   (kindle-store
    brand
    browse-node
    condition
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    item-page
    keywords
    sort
    title
    )
   (kitchen
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    )
   (magazines
    brand
    browse-node
    condition
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    item-page
    keywords
    sort
    state
    title
    )
   (merchants
    brand
    browse-node
    condition
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    item-page
    keywords
    sort
    state
    title
    )
   (miscellaneous
    brand
    browse-node
    condition
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    item-page
    keywords
    sort
    state
    title
    )
   (music
    author
    brand
    browse-node
    condition
    count
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    title
    )
   (musicalinstruments
    brand
    browse-node
    condition
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    item-page
    keywords
    sort
    state
    text-stream
    title
    )
   (musictracks
    keywords
    title
    browse-node
    artist
    author
    actor
    director
    audience-rating
    manufacturer
    music-label
    composer
    publisher
    brand
    conductor
    city
    neighborhood
    orchestra
    text-stream
    )
   (officeproducts
    brand
    browse-node
    condition
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    item-page
    keywords
    sort
    state
    title
    )
   (outdoorliving
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    title
    )
   (pchardware
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    title
    )
   (petsupplies
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   (photo
    actor
    browse-node
    condition
    director
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    mpaa-rating
    publisher
    sort
    state
    text-stream
    title
    )
   (silver-merchants
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (software
    actor
    audience-rating
    availability
    browse-node
    count
    director
    format
    item-page
    keywords
    magazines
    performer
    postal-code
    publisher
    sort
    state
    title
    )
   (sportinggoods
    brand
    browse-node
    condition
    item-page
    keywords
    manufacturer
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    text-stream
    title
    )
   (tools
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    state
    title
    )
   (toys
    browse-node
    condition
    item-page
    keywords
    maximum-price
    merchant-id
    minimum-price
    sort
    title
    )
   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A2S API Reference > BrowseNode IDs
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-syntax define-aws-browse-node-ids
  (syntax-rules ()
    ((_ (locale ...) (index a ...) ...)
     (begin
       '(locale ...)
       '(index a ...)
       ...))))

(define-aws-browse-node-ids
  ;; locales --------------------------------------------------------------
  (ca de fr jp uk us)
  ;; indices ------------------------------------------------------------------------
  (all #f #f #f #f #f #f)
  (apparel #f 16435121 #f 361245011 319532011 1036682)
  (automotive #f #f #f #f #f 15690151)
  (baby #f #f #f #f 60033031 165797011)
  (beauty #f #f #f #f #f 11055981)
  (books 927726 541686 301130 465610 1025612 1000)
  (classical 962454 542676 301164 562034 520920 301668)
  (digital-music #f #f #f #f #f 301668)
  (dvd 952768 547664 409392 562004 283926 130)
  (electronics #f 578122 13910671 3510411 560800 13900851)
  (foreign-books 927726 52557011 69633011 465610 #f #f)
  (gourmet-food #f #f #f #f #f 3580501)
  (grocery #f #f #f #f #f 3580501)
  (health-personal-care #f 569604 #f 161669011 11052591 3760931)
  (hobbies #f #f #f 13331821 #f #f)
  (home-garden #f 3312411 #f #f 11052591 1063498)
  (industrial #f #f #f #f #f 228013)
  (jewelry #f #f #f #f #f 3880591)
  (kitchen #f 3169011 57988031 11717941 11052591 1063498)
  (magazines #f 1198526 #f #f #f 604876)
  (merchants #f #f #f #f #f 44263011)
  (miscellaneous #f #f #f #f #f 13900851)
  (music 962454 542676 301164 562032 520920 301668)
  (musical-instruments #f #f #f #f #f 493964)
  (office-products #f #f #f #f #f 1063498)
  (outdoor-living #f 10925241 #f #f 11052591 13900831)
  (pc-hardware #f 569604 #f #f #f 493964)
  (pet-supplies #f #f #f #f #f 1063498)
  (photo #f 578122 #f #f #f 13900861)
  (software 3234171 3668451 547972 637630 560800 11846801)
  (software-video-games 110218011 541708 548014 #f 1025616 #f)
  (sporting-goods #f 16435121 #f 361245011 319532011 3375301)
  (tools #f #f #f #f 11052591 228013)
  (toys #f 12950661 #f 13331821 595312 130)
  (vhs 962072 547664 301176 561972 573398 404272)
  (video 952768 547664 409392 562002 283926 130)
  (video-games 110218011 578122 548014 1060426 1025616 11846801)
  (watches #f 327473011 60937031 15740941 328229011 #f)
  (wireless #f #f #f #f #f 408234)
  (wireless-accessories #f #f #f #f #f 408234)
  )

(provide "aws")
