(define-module typo.sql
  (use srfi-1)
  (use srfi-13)
  (use text.sql)
  (use text.tree)
  (use util.list)
  (export
   typo-underscore
   typo-sql-where
   typo-sql-select
   typo-sql-count
   typo-sql-insert
   typo-sql-insert-from-port
   typo-sql-update
   typo-sql-delete
   typo-sql-nextval
   typo-sql-create-table
   typo-sql-create-sequence
   typo-sql-drop-table
   typo-sql-drop-sequence
   typo-sql-drop-view
   ))

(select-module typo.sql)

(define (typo-underscore name)
  (regexp-replace-all #/-/ (string-downcase (x->string name)) "_"))

(define (call-with-columns-and-components args proc . options)
  (let-keywords options ((order reverse))
    (let lp ((rest args)
             (count 0)
             (columns '())
             (components '()))
      (if (null? rest)
          (proc count (order columns) (order components))
          (lp (cddr rest) (+ count 1) (cons (car rest) columns) (cons (cadr rest) components))))))

(define (typo-sql-where . args)
  (case (length args)
    ((0) (values "" '()))
    ((1) (values "WHERE id = ?" args))
    (else
     (call-with-columns-and-components
      args
      (lambda (_ columns components)
        (let lp ((columns columns)
                 (components components)
                 (where '())
                 (stuff '()))
          (if (null? columns)
              (values (tree->string `("WHERE " ,@(intersperse " AND " where))) stuff)
              (let ((x (car columns))
                    (y (car components)))
                (if y
                    (lp (cdr columns) (cdr components) `((,x . " = ?") ,@where) (cons y stuff))
                    (lp (cdr columns) (cdr components) `((,x . " IS NULL") ,@where) stuff))))))
      :order identity))))

(define (call-with-where args proc)
  (call-with-values (cut apply typo-sql-where args) proc))

(define (typo-sql-select table names . args)
  (call-with-where
   args
   (lambda (where stuff)
     (values (tree->string `("SELECT "
                             ,(if (null? names) "*" (intersperse ", " names))
                             " FROM " ,(typo-underscore table) " " ,where))
             stuff))))

(define (typo-sql-count table . args)
  (call-with-where
   args
   (lambda (where stuff)
     (values (tree->string `("SELECT count(*) FROM " ,(typo-underscore table) " " ,where))
             stuff))))

(define (typo-sql-insert table . args)
  (call-with-columns-and-components
   args
   (lambda (count columns components)
     (values (tree->string `("INSERT INTO " ,(typo-underscore table) " ("
                             ,@(intersperse ", " columns)
                             ") VALUES (" ,@(intersperse ", " (make-list count #\?)) ")"))
             components))))

(define (typo-sql-insert-from-port table iport)
  (let lp ((statements '())
           (stuffs '()))
    (let ((sexp (read iport)))
      (cond ((eof-object? sexp)
             (values (reverse statements)
                     (reverse stuffs)))
            ((< (length sexp) 2)
             (error "a list is too short:" sexp))
            (else
             (call-with-values
                 (cut apply typo-sql-insert table (if (even? (length sexp)) sexp (cdr sexp)))
               (lambda (statement stuff)
                 (lp (cons statement statements)
                     (cons stuff stuffs)))))))))

(define (typo-sql-update table args . condition)
  (call-with-where
   condition
   (lambda (where stuff)
     (call-with-columns-and-components
      args
      (lambda (_ columns components)
        (let lp ((columns columns)
                 (components components)
                 (sql '())
                 (params '()))
          (if (null? columns)
              (values (tree->string `("UPDATE " ,(typo-underscore table) " SET " ,@(intersperse ", " sql) " " ,where))
                      (append params stuff))
              (let ((x (car columns))
                    (y (car components)))
                (if y
                    (lp (cdr columns) (cdr components) `((,x . " = ?") ,@sql) (cons y params))
                    (lp (cdr columns) (cdr components) `((,x . " = NULL") ,@sql) params))))))
      :order identity))))

(define (typo-sql-delete table . args)
  (call-with-where
   args
   (lambda (where stuff)
     (values (tree->string `("DELETE FROM " ,(typo-underscore table) " " ,where))
             stuff))))

(define (typo-sql-nextval sequence)
  (values "SELECT nextval(?)"
          `(,sequence)))

(define (data-type class)
  (if (pair? class)
      `(,(data-type (car class)) #\( ,(cadr class) #\))
      (case class
        ((<boolean>) 'boolean)
        ((<string>) 'varchar)
        ((<integer>) 'integer)
        (else
         (error "unknown data type:" class)))))

(define (column-constraint args)
  (let-keywords args ((null #t)
                      (unique #f)
                      (default #f))
    (values
     (filter
      identity
      (list
       (and (not null) " NOT NULL")
       (and unique " UNIQUE")
       (and default " DEFAULT ?")))
     (if default (list default) '()))))

(define (table-constraint key clause)
  (let ((tree
         (case key
           ((:unique)
            `("UNIQUE (" ,@(intersperse ", " (map typo-underscore clause)) ")"))
           ((:foreign)
            `("FOREIGN KEY ("
              ,@(intersperse ", " (map typo-underscore (first clause)))
              ") REFERENCES " ,(typo-underscore (second clause)) " ("
              ,@(intersperse ", " (map typo-underscore third clause))
              ")"))
           (else
            (error "unknown table constraint: " (cons key clause))
            ))))
    (values `(,tree) '())))

(define (column-entry-3 key class rest)
  (call-with-values
      (cut column-constraint rest)
    (lambda (tree components)
      (values `(,(typo-underscore key) #\space ,(data-type class) ,@tree)
              components))))

(define (column-entry-2 key val)
  (case key
    ((:unique :foreign)
     (table-constraint key val))
    (else
     (let ((class (car val)))
       (cond ((pair? class)
              (case (car class)
                ((:foreign)
                 (call-with-values
                     (cut column-entry-3 key '<integer> (cdr val))
                   (lambda (tree components)
                     (values `((,@tree " REFERENCES " ,(typo-underscore key) #\(
                                       ,(case (length class)
                                          ((1) `id)
                                          ((2) (typo-underscore (cadr class)))
                                          (else
                                           (error "too many arguments:" class)))
                                       #\)))
                             components))))
                (else
                 (call-with-values
                     (cut column-entry-3 key class (cdr val))
                   (lambda (tree components)
                     (values (list tree) components))))))
             (else
              (case class
                ((:foreign)
                 (call-with-values
                     (cut column-entry-3 (format #f "~a-id" key) '<integer> (cdr val))
                   (lambda (tree components)
                     (values `((,@tree " REFERENCES " ,(typo-underscore key) "(id)"))
                             components))))
                ((:polymorphic)
                 (call-with-values
                     (cut column-entry-3 (format #f "~a-id" key) '<integer> (cdr val))
                   (lambda (t0 c0)
                     (call-with-values
                         (cut column-entry-3 (format #f "~a-type" key) '<string> (cdr val))
                       (lambda (t1 c1)
                         (values (list t0 t1)
                                 (append c0 c1)))))))
                (else
                 (call-with-values
                     (cut column-entry-3 key class (cdr val))
                   (lambda (tree components)
                     (values (list tree) components)))))))))))

(define (list->columns args)
  (call-with-columns-and-components
   args
   (lambda (_ names constraints)
     (let lp ((names names)
              (constraints constraints)
              (tree '())
              (components '()))
       (if (null? names)
           (values tree components)
           (let ((name (car names))
                 (constraint (car constraints)))
             (call-with-values
                 (cut column-entry-2 name constraint)
               (lambda (t c)
                 (lp (cdr names)
                     (cdr constraints)
                     (append tree t)
                     (append components c))))))))))

(define (typo-sql-create-table table . args)
  (call-with-values
      (cut list->columns args)
    (lambda (tree components)
      (values (tree->string `("CREATE TABLE " ,(typo-underscore table) " ("
                              ,@(intersperse ", " `("id SERIAL PRIMARY KEY" ,@tree))
                              ")"))
              components))))

(define (typo-sql-create-sequence name)
  (values (tree->string `("CREATE SEQUENCE " ,(typo-underscore name)))
          '()))

(define-syntax define-typo-sql-drop
  (syntax-rules ()
    ((_ name procedure)
     (define (procedure x)
       (values (tree->string `("DROP " name " " ,(typo-underscore x)))
               '())))))

(define-typo-sql-drop TABLE    typo-sql-drop-table)
(define-typo-sql-drop SEQUENCE typo-sql-drop-sequence)
(define-typo-sql-drop VIEW     typo-sql-drop-view)

(provide "typo/sql")
