%% -*- coding: utf-8 -*-
%% http://www.r6rs.org/final/html/r6rs/r6rs-Z-H-7.html#node_chap_4
%% http://www.r6rs.org/r6rs-errata.html

:- module(sexp,[term_to_sexp/2,sexp_write_string/1]).
:- use_module(library('unicode/unicode_data')).
:- encoding(utf8).

%% Lexeme syntax

lexeme(L) --> sexp:identifier(L),!.
lexeme(L) --> sexp:boolean(L),!.
lexeme(L) --> sexp:number(L),!.
lexeme(L) --> sexp:character(L),!.
lexeme(L) --> sexp:string(L),!.
lexeme(['(']) --> ['('],!.
lexeme([')']) --> [')'],!.
lexeme(['[']) --> ['['],!.
lexeme([']']) --> [']'],!.
lexeme(['#','(']) --> ['#','('],!.
lexeme(['#','v','u','8','(']) --> ['#','v','u','8','('],!.
lexeme(['\'']) --> ['\''],!.
lexeme(['`']) --> ['`'],!.
lexeme([',','@']) --> [',','@'],!.
lexeme([',']) --> [','],!.
lexeme(['.']) --> ['.'],!.
lexeme(['#','\'']) --> ['#','\''],!.
lexeme(['#','`']) --> ['#','`'],!.
lexeme(['#',',','@']) --> ['#',',','@'],!.
lexeme(['#',',']) --> ['#',','].

delimiter(['(']) --> ['('],!.
delimiter([')']) --> [')'],!.
delimiter(['[']) --> ['['],!.
delimiter([']']) --> [']'],!.
delimiter(['"']) --> ['"'],!.
delimiter([';']) --> [';'],!.
delimiter(['#']) --> ['#'],!.
delimiter(D) --> sexp:whitespace(D).

whitespace(['\t']) --> ['\t'],!.
whitespace(['\n']) --> ['\n'],!.
whitespace(['\v']) --> ['\v'],!.
whitespace(['\f']) --> ['\f'],!.
whitespace(['\r']) --> ['\r'],!.
whitespace(['\u0085']) --> ['\u0085'],!. % <next line>
whitespace([WS]) -->
    [WS],
    { char_code(WS,Code),
      unicode_property(Code,general_category(Category)),
      memberchk(Category,['Zs','Zl','Zp']) }.

line_ending(['\n']) --> ['\n'],!.
line_ending(['\u0085']) --> ['\u0085'],!.
line_ending(['\u2028']) --> ['\u2028'],!. % <line separator>
line_ending(['\r','\n']) --> ['\r','\n'],!.
line_ending(['\r','\u0085']) --> ['\r','\u0085'],!.
line_ending(['\r']) --> ['\r'].

up_to_eol(LE) --> sexp:line_ending(LE),!.
up_to_eol(['\u2029']) --> ['\u2029'],!. % <paragraph separator>
up_to_eol([H|T]) --> [H],!,sexp:up_to_eol(T).
up_to_eol([]) --> [].

comment([';'|X]) --> [';'],!,sexp:up_to_eol(X).
comment(C) --> sexp:nested_comment(C),!.
comment(['#',';'|X]) -->
    ['#',';'],!,
    sexp:interlexeme_space(Y),
    sexp:datum(Z),
    { append(Y,Z,X) }.
comment(['#','!','r','6','r','s']) --> ['#','!','r','6','r','s'].

nested_comment(['#','|'|X]) -->
    ['#','|'],
    sexp:comment_text(Y),
    sexp:comment_conts0(Z),
    ['|','#'],
    { append([Y,Z,['|','#']],X) }.

comment_text(X) --> sexp:comment_text(0,X).
comment_text(_,[]) --> [].
comment_text(0,[H|T]) -->
    [H],
    { ( H == '#' -> S = 1
      ; H == '|' -> S = 2
      ; S = 0) },
    comment_text(S,T).
comment_text(1,[H|T]) -->
    [H],
    { ( H == '|' -> fail
      ; H == '#' -> S = 1
      ; S = 0) },
    comment_text(S,T).
comment_text(2,[H|T]) -->
    [H],
    { ( H == '#' -> fail
      ; H == '|' -> S = 2
      ; S = 0) },
    comment_text(S,T).

comment_conts0(X) -->
    sexp:comment_cont(Y),
    sexp:comment_conts0(Z),
    { append(Y,Z,X) }.
comment_conts0([]) --> [].

comment_cont(X) -->
    sexp:nested_comment(Y),
    sexp:comment_text(Z),
    { append(Y,Z,X) }.

atmosphere(X) --> sexp:whitespace(X),!.
atmosphere(X) --> sexp:comment(X).

interlexeme_space(X) -->
    sexp:atmosphere(Y),!,
    interlexeme_space(Z),
    { append(Y,Z,X) }.
interlexeme_space([]) --> [].

identifier(X) -->
    sexp:initial(Y),
    sexp:subsequents(Z),!,
    { append(Y,Z,X) }.
identifier(X) --> sexp:peculiar_identifier(X).

initial(I) --> sexp:constituent(I),!.
initial(I) --> sexp:special_initial(I),!.
initial(I) --> sexp:inline_hex_escape(I).

letter([L]) -->
    [L],
    { char_type(L,ascii),char_type(L,alpha) }.

constituent(C) --> letter(C),!.
constituent([C]) -->
    [C],
    { char_code(C,Code),
      127 < Code,
      unicode_property(Code,general_category(Category)),
      memberchk(Category,['Lu','Ll','Lt','Lm','Lo','Mn','Nl','No','Pd','Pc','Po','Sc','Sm','Sk','So','Co']) }.

special_initial(['!']) --> ['!'],!.
special_initial(['$']) --> ['$'],!.
special_initial(['%']) --> ['%'],!.
special_initial(['&']) --> ['&'],!.
special_initial(['*']) --> ['*'],!.
special_initial(['/']) --> ['/'],!.
special_initial([':']) --> [':'],!.
special_initial(['<']) --> ['<'],!.
special_initial(['=']) --> ['='],!.
special_initial(['>']) --> ['>'],!.
special_initial(['?']) --> ['?'],!.
special_initial(['^']) --> ['^'],!.
special_initial(['_']) --> ['_'],!.
special_initial(['~']) --> ['~'].

subsequent(S) --> sexp:initial(S),!.
subsequent(S) --> sexp:digit(S),!.
subsequent(S) -->
    [S],
    { char_code(S,Code),
      unicode_property(Code,general_category(Category)),
      memberchk(Category,['Nd','Mc','Me']) },!.
subsequent(S) --> sexp:special_subsequent(S).

subsequents(X) -->
    sexp:subsequent(Y),!,
    sexp:subsequents(Z),
    { append(Y,Z,X) }.
subsequents([]) --> [].

digit([D]) --> [D],{ char_type(D,digit) }.

hex_digit([HD]) --> [HD],{ char_type(HD,xdigit(_)) }.

special_subsequent(['+']) --> ['+'],!.
special_subsequent(['-']) --> ['-'],!.
special_subsequent(['.']) --> ['.'],!.
special_subsequent(['@']) --> ['@'].

inline_hex_escape_initial(['\\','x']) --> ['\\','x'].

inline_hex_escape(X) -->
    sexp:inline_hex_escape_initial(Y),!,
    sexp:hex_scalar_value(Z),
    [';'],
    { append([Y,Z,[';']],X) }.

hex_scalar_value([H|T]) --> sexp:hex_digit([H]),sexp:hex_scalar_value(T).
hex_scalar_value([]) --> [].

peculiar_identifier(['+']) --> ['+'],!.
peculiar_identifier(['.','.','.']) --> ['.','.','.'],!.
peculiar_identifier(['-','>'|X]) -->
    ['-','>'],!,
    sexp:subsequents(X).
peculiar_identifier(['-']) --> ['-'].

boolean(['#','t']) --> ['#','t'],!.
boolean(['#','T']) --> ['#','T'],!.
boolean(['#','f']) --> ['#','f'],!.
boolean(['#','F']) --> ['#','F'].

character(['#','\\'|X]) --> ['#','\\'],sexp:character_name(X),!.
character(['#','\\','x'|X]) --> ['#','\\','x'],sexp:hex_scalar_value(X),!.
character(['#','\\',X]) --> ['#','\\',X].

character_name(['n','u','l']) --> ['n','u','l'],!.
character_name(['a','l','a','r','m']) --> ['a','l','a','r','m'],!.
character_name(['b','a','c','k','s','p','a','c','e']) --> ['b','a','c','k','s','p','a','c','e'],!.
character_name(['t','a','b']) --> ['t','a','b'],!.
character_name(['l','i','n','e','f','e','e','d']) --> ['l','i','n','e','f','e','e','d'],!.
character_name(['n','e','w','l','i','n','e']) --> ['n','e','w','l','i','n','e'],!.
character_name(['v','t','a','b']) --> ['v','t','a','b'],!.
character_name(['p','a','g','e']) --> ['p','a','g','e'],!.
character_name(['r','e','t','u','r','n']) --> ['r','e','t','u','r','n'],!.
character_name(['e','s','c']) --> ['e','s','c'],!.
character_name(['s','p','a','c','e']) --> ['s','p','a','c','e'],!.
character_name(['d','e','l','e','t','e']) --> ['d','e','l','e','t','e'].

string(X) -->
    ['"'],
    sexp:string_elements0(Y),
    ['"'],!,
    { append([['"'],Y,['"']],X) }.

string_elements0(X) -->
    sexp:string_element(Y),!,
    sexp:string_elements0(Z),
    { append(Y,Z,X) }.
string_elements0([]) --> [].

string_element([C]) --> [C],{ C \== '\\',C \== '"' },!.
string_element(['\\','a']) --> ['\\','a'],!.
string_element(['\\','b']) --> ['\\','b'],!.
string_element(['\\','t']) --> ['\\','t'],!.
string_element(['\\','n']) --> ['\\','n'],!.
string_element(['\\','v']) --> ['\\','v'],!.
string_element(['\\','f']) --> ['\\','f'],!.
string_element(['\\','r']) --> ['\\','r'],!.
string_element(['\\','"']) --> ['\\','"'],!.
string_element(['\\','\\']) --> ['\\','\\'],!.
string_element(X) -->
    ['\\'],
    sexp:intraline_whitespaces0(W0), % R6RS has the missing Kleene star.
    sexp:line_ending(LE),
    sexp:intraline_whitespaces0(W1), % ditto
    { append([['\\'],W0,LE,W1],X) },!.
string_element(X) --> sexp:inline_hex_escape(X).

intraline_whitespace(['\t']) --> ['\t'],!.
intraline_whitespace([IW]) -->
    [IW],
    { char_code(IW,Code),
      unicode_property(Code,general_category('Zs')) }.

intraline_whitespaces0([C|Rest]) -->
    sexp:intraline_whitespace([C]),!,
    sexp:intraline_whitespaces0(Rest).
intraline_whitespaces0([]) --> [].

%% A <hex scalar value> represents a Unicode scalar value between 0 and #x10FFFF,
%% excluding the range [#xD800, #xDFFF].

number(N) --> sexp:num(2,N),!.
number(N) --> sexp:num(8,N),!.
number(N) --> sexp:num(10,N),!.
number(N) --> sexp:num(16,N).

num(R,N) -->
    sexp:prefix(R,P),
    sexp:complex(R,C),
    { append(P,C,N) }.

proper_sign(['+']) --> ['+'],!.
proper_sign(['-']) --> ['-'].

complex(R,C) --> sexp:real(R,C).
complex(R,C) -->
    sexp:real(R,D),
    ['@'],!,
    sexp:real(R,E),
    { append([D,['@'],E],C) }.
complex(R,C) -->
    sexp:real(R,D),
    sexp:proper_sign(Op),
    sexp:ureal(R,E),
    ['i'],
    { append([D,Op,E,['i']],C) }.
complex(R,C) -->
    sexp:real(R,D),
    sexp:proper_sign(Op),
    sexp:naninf(E),
    ['i'],
    { append([D,Op,E,['i']],C) }.
complex(R,C) -->
    sexp:real(R,D),
    sexp:proper_sign(Op),
    ['i'],
    { append([D,Op,['i']],C) }.
complex(R,C) -->
    sexp:proper_sign(Op),
    sexp:ureal(R,D),
    ['i'],
    { append([Op,D,['i']],C) }.
complex(_,C) -->
    sexp:naninf(D),
    ['i'],
    { append(D,['i'],C) }.
complex(_,C) -->
    sexp:proper_sign(D),
    ['i'],
    { append(D,['i'],C) }.

real(_,X) -->
    sexp:proper_sign(Y),
    sexp:naninf(Z),!,
    { append(Y,Z,X) }.
real(R,X) -->
    sexp:sign(Y),
    sexp:ureal(R,Z),
    { append(Y,Z,X) }.

naninf(['n','a','n','.','0']) --> ['n','a','n','.','0'],!.
naninf(['i','n','f','.','0']) --> ['i','n','f','.','0'].

ureal(R,X) -->
    sexp:uinteger(R,Y),
    ['/'],!,
    sexp:uinteger(R,Z),
    { append([Y,['/'],Z],X) }.
ureal(R,X) -->
    sexp:decimal(R,Y),!,
    sexp:mantissa_width(Z),
    { append(Y,Z,X) }.
ureal(R,X) --> sexp:uinteger(R,X).

decimal(10,D) -->
    ['.'],!,
    sexp:digits1(10,DS),
    sexp:suffix(S),
    { append([['.'],DS,S],D) }.
decimal(10,D) -->
    sexp:digits1(10,DS1),
    ['.'],!,
    sexp:digits0(10,DS0),
    sexp:suffix(S),
    { append([DS1,['.'],DS0,S],D) }.
decimal(10,D) -->
    sexp:uinteger(10,U),
    sexp:suffix(S),
    { append(U,S,D) }.

uinteger(R,U) --> sexp:digits1(R,U).

prefix(R,P) -->
    sexp:radix(R,X),
    sexp:exactness(E),
    { append(X,E,P) }.
prefix(R,P) -->
    sexp:exactness(E),
    sexp:radix(R,X),
    { append(X,E,P) }.

suffix(X) -->
    sexp:exponent_marker(EM),!,
    sexp:sign(S),
    sexp:digits1(10,D),
    { append([EM,S,D],X) }.
suffix([]) --> [].

exponent_marker(['e']) --> ['e'],!.
exponent_marker(['E']) --> ['E'],!.
exponent_marker(['s']) --> ['s'],!.
exponent_marker(['S']) --> ['S'],!.
exponent_marker(['f']) --> ['f'],!.
exponent_marker(['F']) --> ['F'],!.
exponent_marker(['d']) --> ['d'],!.
exponent_marker(['D']) --> ['D'],!.
exponent_marker(['l']) --> ['l'],!.
exponent_marker(['L']) --> ['L'].

mantissa_width(['|'|D]) -->
    ['|'],!,
    sexp:digits1(10,D).
mantissa_width([]) --> [].

sign(S) --> sexp:proper_sign(S),!.
sign([]) --> [].

exactness(['#','i']) --> ['#','i'],!.
exactness(['#','I']) --> ['#','I'],!.
exactness(['#','e']) --> ['#','e'],!.
exactness(['#','E']) --> ['#','E'],!.
exactness([]) --> [].

radix(2,['#','b']) --> ['#','b'],!.
radix(2,['#','B']) --> ['#','B'],!.
radix(8,['#','o']) --> ['#','o'],!.
radix(8,['#','O']) --> ['#','O'],!.
radix(10,['#','d']) --> ['#','d'],!.
radix(10,['#','D']) --> ['#','D'],!.
radix(10,[]) --> [].
radix(16,['#','x']) --> ['#','x'],!.
radix(16,['#','X']) --> ['#','X'],!.

digit(2,['0']) --> ['0'],!.
digit(2,['1']) --> ['1'].

digit(8,['0']) --> ['0'],!.
digit(8,['1']) --> ['1'],!.
digit(8,['2']) --> ['2'],!.
digit(8,['3']) --> ['3'],!.
digit(8,['4']) --> ['4'],!.
digit(8,['5']) --> ['5'],!.
digit(8,['6']) --> ['6'],!.
digit(8,['7']) --> ['7'].

digit(10,D) --> sexp:digit(D).
digit(16,D) --> sexp:hex_digit(D).

digits0(10,DS) --> digits1(10,DS),!.
digits0(10,[]) --> [].

digits1(10,[D|S]) -->
    sexp:digit(10,[D]),!,
    sexp:digits0(10,S).

%% Datum syntax

data0(DS) --> sexp:data1(DS),!.
data0([]) --> [].

data1(DS) -->
    sexp:datum(D),
    sexp:interlexeme_space(X),
    sexp:data0(Res),
    { append([D,X,Res],DS) }.

datum(D) --> sexp:lexeme_datum(D),!.
datum(D) --> sexp:compound_datum(D).

lexeme_datum(LD) --> sexp:boolean(LD),!.
lexeme_datum(LD) --> sexp:number(LD),!.
lexeme_datum(LD) --> sexp:character(LD),!.
lexeme_datum(LD) --> sexp:string(LD),!.
lexeme_datum(LD) --> sexp:symbol(LD).

symbol(S) --> sexp:identifier(S).

compound_datum(CD) --> sexp:list(CD),!.
compound_datum(CD) --> sexp:vector(CD),!.
compound_datum(CD) --> sexp:bytevector(CD).

list(CD) -->
    ['('],
    sexp:interlexeme_space(X),
    sexp:data0(D0),
    sexp:interlexeme_space(Y),
    [')'],!,
    { append([['('],X,D0,Y,[')']],CD) }.
list(CD) -->
    ['['],
    sexp:interlexeme_space(X),
    sexp:data0(D0),
    sexp:interlexeme_space(Y),
    [']'],!,
    { append([['['],X,D0,Y,[']']],CD) }.
list(CD) -->
    ['('],
    sexp:interlexeme_space(X1),
    sexp:data1(D1),
    sexp:interlexeme_space(Y1),
    ['.'],
    sexp:interlexeme_space(X),
    sexp:datum(D),
    sexp:interlexeme_space(Y),
    [')'],!,
    { append([['('],X1,D1,Y1,['.'],X,D,Y,[')']],CD) }.
list(CD) -->
    ['['],
    sexp:interlexeme_space(X1),
    sexp:data1(D1),
    sexp:interlexeme_space(Y1),
    ['.'],
    sexp:interlexeme_space(X),
    sexp:datum(D),
    sexp:interlexeme_space(Y),
    [']'],!,
    { append([['['],X1,D1,Y1,['.'],X,D,Y,[']']],CD) }.
list(CD) --> sexp:abbreviation(CD).

abbreviation(A) -->
    sexp:abbrev_prefix(AP),!,
    sexp:interlexeme_space(X),
    sexp:datum(D),
    { append([AP,X,D],A) }.

abbrev_prefix(['\'']) --> ['\''],!.
abbrev_prefix(['`']) --> ['`'],!.
abbrev_prefix([',']) --> [','],!.
abbrev_prefix([',','@']) --> [',','@'],!.
abbrev_prefix(['#','\'']) --> ['#','\''],!.
abbrev_prefix(['#','`']) --> ['#','`'],!.
abbrev_prefix(['#',',','@']) --> ['#',',','@'],!.
abbrev_prefix(['#',',']) --> ['#',','].

vector(V) -->
    ['#', '('],
    sexp:interlexeme_space(X),
    sexp:data0(D),
    sexp:interlexeme_space(Y),
    [')'],!,
    { append([['#','('],X,D,Y,[')']],V) }.

bytevector(BV) -->
    ['#','v','u','8','('],
    sexp:interlexeme_space(X),
    sexp:u8s0(US),
    sexp:interlexeme_space(Y),
    [')'],!,
    { append([['#','v','u','8','('],X,US,Y,[')']],BV) }.

u8(U8) --> sexp:number(U8). %% bother to check an exact integer in {0, ..., 255}.

u8s0(U8S) -->
    u8(U8),!,
    interlexeme_space(X),
    u8s0(Rest),
    { append([U8,X,Rest],U8S) }.
u8s0([]) --> [].

%% Translation of Term

term_to_sexp(true,['#','t']) :- !.
term_to_sexp(false,['#','f']) :- !.

term_to_sexp(Integer,Chars) :- integer(Integer),!,number_chars(Integer,Chars).
term_to_sexp(Float,Chars) :- float(Float),!,number_chars(Float,Chars).
term_to_sexp(Rational,Chars) :-
    rational(Rational,N,D),!,
    number_chars(N,NC),
    number_chars(D,DC),
    append([NC,['/'],DC],Chars).

term_to_sexp([],['(',')']) :- !.
term_to_sexp([X],S) :- !,
    term_to_sexp(X,S_X),
    append(['('|S_X],[')'],S).
term_to_sexp([H|T],S) :- !,
    term_to_sexp(H,S_H),
    term_to_sexp(T,[P|R]),
    append([P|S_H],[' '|R],S).

term_to_sexp(Atom,Chars) :- atom(Atom),!,atom_chars(Atom,Chars).

term_to_sexp(C,S) :-
    compound(C),!,
    C =.. List,
    term_to_sexp(List,S).

%% sexp_write_string

sexp_write_string(Chars) :-
    is_list(Chars),!,
    write('"'),
    sexp_write_string_chars(Chars),
    write('"').
sexp_write_string(Atom) :-
    atom(Atom),!,
    atom_chars(Atom,Chars),
    sexp_write_string(Chars).
sexp_write_string(Term) :-
    term_to_atom(Term,Atom),!,
    sexp_write_string(Atom).

sexp_write_string_chars([]) :- !.
sexp_write_string_chars([C|Rest]) :-
    sexp_write_string_char(C),
    sexp_write_string_chars(Rest).

sexp_write_string_char('\\') :- !,write('\\\\').
sexp_write_string_char('"') :- !,write('\\"').
sexp_write_string_char('\n') :- !,write('\\n').
sexp_write_string_char('\t') :- !,write('\\t').
sexp_write_string_char('\r') :- !,write('\\r').
sexp_write_string_char('\f') :- !,write('\\f').
sexp_write_string_char(C) :- put_char(C).
